@@ -44,6 +44,8 @@ Reference
    :inherited-members:
 """
 
+from __future__ import print_function
+
 import sys
 import numpy as np
 
@@ -52,22 +54,45 @@ from guidata.qt.QtGui import (QSizePolicy, QColor, QPixmap, QPrinter,
 from guidata.qt.QtCore import QSize, Qt
 
 from guidata.configtools import get_font
+from guidata.py3compat import to_text_string, is_text_string, maxsize
 
 # Local imports
 from guiqwt.transitional import (QwtPlot, QwtLinearScaleEngine,
                                  QwtLog10ScaleEngine, QwtText, QwtPlotCanvas)
+from guiqwt import io
 from guiqwt.config import CONF, _
 from guiqwt.events import StatefulEventFilter
 from guiqwt.interfaces import IBasePlotItem, IItemType, ISerializableType
-from guiqwt.styles import ItemParameters, AxeStyleParam, AxesParam
+from guiqwt.styles import ItemParameters, AxeStyleParam, AxesParam, AxisParam
 from guiqwt.signals import (SIG_ITEMS_CHANGED, SIG_ACTIVE_ITEM_CHANGED,
                             SIG_ITEM_SELECTION_CHANGED, SIG_ITEM_MOVED,
-                            SIG_PLOT_LABELS_CHANGED)
+                            SIG_PLOT_LABELS_CHANGED, SIG_ITEM_REMOVED)
+
+
+#==============================================================================
+# Utilities for plot items
+#==============================================================================
+
+def canvas_to_axes(item, pos):
+    """Convert (x,y) from canvas coordinates system to axes coordinates"""
+    plot, ax, ay = item.plot(), item.xAxis(), item.yAxis()
+    return plot.invTransform(ax, pos.x()), plot.invTransform(ay, pos.y())
+
+def axes_to_canvas(item, x, y):
+    """Convert (x,y) from axes coordinates to canvas coordinates system"""
+    plot, ax, ay = item.plot(), item.xAxis(), item.yAxis()
+    return plot.transform(ax, x), plot.transform(ay, y)
+
+
+
+#==============================================================================
+# Base plot widget
+#==============================================================================
 
 PARAMETERS_TITLE_ICON = {
                          'grid': (_("Grid..."), "grid.png" ),
                          'axes': (_("Axes style..."), "axes.png" ),
-                         'item': (_("Parameters..."),"settings.png" ),
+                         'item': (_("Parameters..."), "settings.png" ),
                          }
     
 
@@ -94,6 +119,8 @@ class BasePlot(QwtPlot):
     AXIS_TYPES = {"lin" : QwtLinearScaleEngine,
                   "log" : QwtLog10ScaleEngine }
     AXIS_CONF_OPTIONS = ("axis", "axis", "axis", "axis")
+    DEFAULT_ACTIVE_XAXIS = X_BOTTOM
+    DEFAULT_ACTIVE_YAXIS = Y_LEFT
 
     def __init__(self, parent=None, section="plot"):
         super(BasePlot, self).__init__(parent)
@@ -105,27 +132,35 @@ class BasePlot(QwtPlot):
         self.items = []
         self.active_item = None
         self.last_selected = {} # a mapping from item type to last selected item
-        self.axes_styles = [AxeStyleParam(_(u"Left")),
-                            AxeStyleParam(_(u"Right")),
-                            AxeStyleParam(_(u"Bottom")),
-                            AxeStyleParam(_(u"Top"))]
-        self._active_xaxis = self.X_BOTTOM
-        self._active_yaxis = self.Y_LEFT
+        self.axes_styles = [AxeStyleParam(_("Left")),
+                            AxeStyleParam(_("Right")),
+                            AxeStyleParam(_("Bottom")),
+                            AxeStyleParam(_("Top"))]
+        self._active_xaxis = self.DEFAULT_ACTIVE_XAXIS
+        self._active_yaxis = self.DEFAULT_ACTIVE_YAXIS
         self.read_axes_styles(section, self.AXIS_CONF_OPTIONS)
         self.font_title = get_font(CONF, section, "title")
         canvas = self.canvas()
         canvas.setFocusPolicy(Qt.StrongFocus)
         canvas.setFocusIndicator(QwtPlotCanvas.ItemFocusIndicator)
         self.connect(self, SIG_ITEM_MOVED, self._move_selected_items_together)
-        
-    #---- QwtPlot API ----------------------------------------------------------
+
+    #---- QWidget API ---------------------------------------------------------
+    def mouseDoubleClickEvent(self, event):
+        """Reimplement QWidget method"""
+        for axis_id in self.AXIS_IDS:
+            widget = self.axisWidget(axis_id)
+            if widget.geometry().contains(event.pos()):
+                self.edit_axis_parameters(axis_id)
+
+    #---- QwtPlot API ---------------------------------------------------------
     def showEvent(self, event):
         """Reimplement Qwt method"""
         QwtPlot.showEvent(self, event)
         if self._start_autoscaled:
             self.do_autoscale()
 
-    #---- Public API -----------------------------------------------------------
+    #---- Public API ----------------------------------------------------------
     def _move_selected_items_together(self, item, x0, y0, x1, y1):
         """Selected items move together"""
         for selitem in self.get_selected_items():
@@ -143,7 +178,7 @@ class BasePlot(QwtPlot):
         
     def get_title(self):
         """Get plot title"""
-        return unicode(self.title().text())
+        return to_text_string(self.title().text())
 
     def set_title(self, title):
         """Set plot title"""
@@ -216,7 +251,7 @@ class BasePlot(QwtPlot):
         color: color name (string) or QColor instance
         """
         axis_id = self.get_axis_id(axis_id)
-        if isinstance(color, basestring):
+        if is_text_string(color):
             color = QColor(color)
         self.axes_styles[axis_id].color = str(color.name())
         self.update_axis_style(axis_id)
@@ -274,17 +309,18 @@ class BasePlot(QwtPlot):
         """Return the name ('lin' or 'log') of the scale used by axis"""
         axis_id = self.get_axis_id(axis_id)
         engine = self.axisScaleEngine(axis_id)
-        for axis_label, axis_type in self.AXIS_TYPES.items():
+        for axis_label, axis_type in list(self.AXIS_TYPES.items()):
             if isinstance(engine, axis_type):
                 return axis_label
         return "lin"  # unknown default to linear
 
-    def set_axis_scale(self, axis_id, scale):
+    def set_axis_scale(self, axis_id, scale, autoscale=True):
         """Set axis scale
         Example: self.set_axis_scale(curve.yAxis(), 'lin')"""
         axis_id = self.get_axis_id(axis_id)
         self.setAxisScaleEngine(axis_id, self.AXIS_TYPES[scale]())
-        self.do_autoscale(replot=False)
+        if autoscale:
+            self.do_autoscale(replot=False)
 
     def get_scales(self):
         """Return active curve scales"""
@@ -352,7 +388,7 @@ class BasePlot(QwtPlot):
             
     def save_widget(self, fname):
         """Grab widget's window and save it to filename (*.png, *.pdf)"""
-        fname = unicode(fname)
+        fname = to_text_string(fname)
         if fname.lower().endswith('.pdf'):
             printer = QPrinter()
             printer.setOutputFormat(QPrinter.PdfFormat)
@@ -397,7 +433,7 @@ class BasePlot(QwtPlot):
         else:
             item.setZ(self.get_max_z()+1)
         if item in self.items:
-            print >>sys.stderr, "Warning: item %r is already attached to plot" % item
+            print("Warning: item %r is already attached to plot" % item, file=sys.stderr)
         else:
             self.items.append(item)
         self.emit(SIG_ITEMS_CHANGED, self)
@@ -414,16 +450,17 @@ class BasePlot(QwtPlot):
         else:
             z = zlist[dzlist]+1
         self.add_item(item, z=z)
-        
-    def del_item(self, item):
-        """
-        Remove item from widget
-        Convenience function (see 'del_items')
-        """
-        try:
-            self.del_items([item])
-        except ValueError:
-            raise ValueError, "item not in plot"
+
+    def __clean_item_references(self, item):
+        """Remove all reference to this item (active,
+        last_selected"""
+        if item is self.active_item:
+            self.active_item = None
+            self._active_xaxis = self.DEFAULT_ACTIVE_XAXIS
+            self._active_yaxis = self.DEFAULT_ACTIVE_YAXIS
+        for key, it in list(self.last_selected.items()):
+            if item is it:
+                del self.last_selected[key]
 
     def del_items(self, items):
         """Remove item from widget"""
@@ -435,10 +472,21 @@ class BasePlot(QwtPlot):
             # raises ValueError if item not in list
             self.items.remove(item)
             self.__clean_item_references(item)
+            self.emit(SIG_ITEM_REMOVED, item)
         self.emit(SIG_ITEMS_CHANGED, self)
         if active_item is not self.get_active_item():
             self.emit(SIG_ACTIVE_ITEM_CHANGED, self)
 
+    def del_item(self, item):
+        """
+        Remove item from widget
+        Convenience function (see 'del_items')
+        """
+        try:
+            self.del_items([item])
+        except ValueError:
+            raise ValueError("item not in plot")
+
     def set_item_visible(self, item, state, notify=True, replot=True):
         """Show/hide *item* and emit a SIG_ITEMS_CHANGED signal"""
         item.setVisible(state)
@@ -492,16 +540,32 @@ class BasePlot(QwtPlot):
         import pickle
         items = pickle.load(iofile)
         for item in items:
-            self.add_item(item, z=item.z())
+            self.add_item(item)
 
-    def __clean_item_references(self, item):
-        """Remove all reference to this item (active,
-        last_selected"""
-        if item is self.active_item:
-            self.active_item = None
-        for key, it in self.last_selected.items():
-            if item is it:
-                del self.last_selected[key]
+    def serialize(self, writer, selected=False):
+        """
+        Save (serializable) items to HDF5 file:
+            * writer: :py:class:`guidata.hdf5io.HDF5Writer` object
+            * selected=False: if True, will save only selected items
+            
+        See also :py:meth:`guiqwt.baseplot.BasePlot.restore_items_from_hdf5`
+        """
+        if selected:
+            items = self.get_selected_items()
+        else:
+            items = self.items[:]
+        items = [item for item in items if ISerializableType in item.types()]
+        io.save_items(writer, items)
+        
+    def deserialize(self, reader):
+        """
+        Restore items from HDF5 file:
+            * reader: :py:class:`guidata.hdf5io.HDF5Reader` object
+            
+        See also :py:meth:`guiqwt.baseplot.BasePlot.save_items_to_hdf5`
+        """
+        for item in io.load_items(reader):
+            self.add_item(item)
 
     def set_items(self, *args):
         """Utility function used to quickly setup a plot
@@ -662,7 +726,7 @@ class BasePlot(QwtPlot):
                            distance to 'pos' is less than close_dist
         else: return the closest item
         """
-        selobj, distance, inside, handle = None, sys.maxint, None, None
+        selobj, distance, inside, handle = None, maxsize, None, None
         for obj in self.get_items(z_sorted=True):
             if not obj.isVisible() or not obj.can_select():
                 continue
@@ -682,7 +746,7 @@ class BasePlot(QwtPlot):
         Return nearest item for which position 'pos' is inside of it
         (iterate over items with respect to their 'z' coordinate)
         """
-        selobj, distance, inside, handle = None, sys.maxint, None, None
+        selobj, distance, inside, handle = None, maxsize, None, None
         for obj in self.get_items(z_sorted=True):
             if not obj.isVisible() or not obj.can_select():
                 continue
@@ -738,7 +802,7 @@ class BasePlot(QwtPlot):
             
     def set_item_parameters(self, itemparams):
         """Set item (plot, here) parameters"""
-        # Axe styles
+        # Axe styles        
         datasets = [itemparams.get("AxeStyleParam%d" % i) for i in range(4)]
         if datasets[0] is not None:
             self.axes_styles = datasets
@@ -758,6 +822,18 @@ class BasePlot(QwtPlot):
         self.get_plot_parameters(key, itemparams)
         title, icon = PARAMETERS_TITLE_ICON[key]
         itemparams.edit(self, title, icon)
+
+    def edit_axis_parameters(self, axis_id):
+        """Edit axis parameters"""
+        if axis_id in (self.Y_LEFT, self.Y_RIGHT):
+            title = _("Y Axis")
+        else:
+            title = _("X Axis")
+        param = AxisParam(title=title)
+        param.update_param(self, axis_id)
+        if param.edit(parent=self):
+            param.update_axis(self, axis_id)
+            self.replot()
         
     def do_autoscale(self, replot=True):
         """Do autoscale on all axes"""
