@@ -56,7 +56,9 @@ Reference
    :members:
 """
 
+import os.path as osp
 from numpy import arange, array, zeros, meshgrid, ndarray
+from guidata.py3compat import is_text_string
 
 # Local imports
 from guiqwt.config import _, CONF, make_title
@@ -77,20 +79,19 @@ from guiqwt.styles import (update_style_attr, CurveParam, ErrorBarParam,
                            LineStyleParam, AnnotationParam, QuadGridParam,
                            LabelParamWithContents, MarkerParam)
 from guiqwt.label import (LabelItem, LegendBoxItem, RangeComputation,
-                          RangeComputation2d, DataInfoLabel,
+                          RangeComputation2d, DataInfoLabel, RangeInfo,
                           SelectedLegendBoxItem)
-import os.path as osp
 
 # default offset positions for anchors
 ANCHOR_OFFSETS = {
-                  "TL" : ( 5,  5),
-                  "TR" : (-5,  5),
-                  "BL" : ( 5, -5),
-                  "BR" : (-5, -5),
-                  "L"  : ( 5,  0),
-                  "R"  : (-5,  0),
-                  "T"  : ( 0,  5),
-                  "B"  : ( 0, -5),
+                  "TL": ( 5,  5),
+                  "TR": (-5,  5),
+                  "BL": ( 5, -5),
+                  "BR": (-5, -5),
+                  "L": ( 5,  0),
+                  "R": (-5,  0),
+                  "T": ( 0,  5),
+                  "B": ( 0, -5),
                   }
 
 CURVE_COUNT = 0
@@ -213,38 +214,44 @@ class PlotItemBuilder(object):
     def __get_arg_triple_plot(self, args):
         """Convert MATLAB-like arguments into x, y, style"""
         def get_x_y_from_data(data):
-            if len(data.shape) == 1 or data.shape[0] == 1 or data.shape[1] == 1:
+            if isinstance(data, (tuple, list)):
+                data = array(data)
+            if len(data.shape) == 1 or 1 in data.shape:
                 x = arange(data.size)
                 y = data
             else:
                 x = arange(len(data[:, 0]))
-                y = [data[:, i] for i in range(len(data[0, :]))]
+                y = [data[:, i] for i in range(len(data[0,:]))]
             return x, y
             
-        if len(args)==1:
-            if isinstance(args[0], basestring):
+        if len(args) == 1:
+            if is_text_string(args[0]):
                 x = array((), float)
                 y = array((), float)
                 style = args[0]
             else:
                 x, y = get_x_y_from_data(args[0])
                 if isinstance(y, ndarray):
-                    style = self.style.next()
+                    style = next(self.style)
                 else:
-                    style = [self.style.next() for yi in y]
-        elif len(args)==2:
+                    style = [next(self.style) for yi in y]
+        elif len(args) == 2:
             a1, a2 = args
-            if isinstance(a2, basestring):
+            if is_text_string(a2):
                 x, y = get_x_y_from_data(a1)
                 style = a2
             else:
                 x = a1
                 y = a2
-                style = self.style.next()
+                style = next(self.style)
         elif len(args)==3:
             x, y, style = args
         else:
             raise TypeError("Wrong number of arguments")
+        if isinstance(x, (list, tuple)):
+            x = array(x)
+        if isinstance(y, (list, tuple)):
+            y = array(y)
         return x, y, style
         
     def __get_arg_triple_errorbar(self, args):
@@ -253,10 +260,10 @@ class PlotItemBuilder(object):
             y, dy = args
             x = arange(len(y))
             dx = zeros(len(y))
-            style = self.style.next()
+            style = next(self.style)
         elif len(args)==3:
             a1, a2, a3 = args
-            if isinstance(a3, basestring):
+            if is_text_string(a3):
                 y, dy = a1, a2
                 x = arange(len(y))
                 dx = zeros(len(y))
@@ -264,16 +271,16 @@ class PlotItemBuilder(object):
             else:
                 x, y, dy = args
                 dx = zeros(len(y))
-                style = self.style.next()
+                style = next(self.style)
         elif len(args)==4:
             a1, a2, a3, a4 = args
-            if isinstance(a4, basestring):
+            if is_text_string(a4):
                 x, y, dy = a1, a2, a3
                 dx = zeros(len(y))
                 style = a4
             else:
                 x, y, dx, dy = args
-                style = self.style.next()
+                style = next(self.style)
         elif len(args)==5:
             x, y, dx, dy, style = args
         else:
@@ -326,7 +333,7 @@ class PlotItemBuilder(object):
         self.__set_curve_axes(curve, xaxis, yaxis)
         return curve
 
-    def curve(self, x, y, title=u"",
+    def curve(self, x, y, title="",
               color=None, linestyle=None, linewidth=None,
               marker=None, markersize=None, markerfacecolor=None,
               markeredgecolor=None, shade=None, fitted=None,
@@ -424,7 +431,7 @@ class PlotItemBuilder(object):
         self.__set_curve_axes(curve, xaxis, yaxis)
         return curve
         
-    def error(self, x, y, dx, dy, title=u"",
+    def error(self, x, y, dx, dy, title="",
               color=None, linestyle=None, linewidth=None,
               errorbarwidth=None, errorbarcap=None, errorbarmode=None,
               errorbaralpha=None, marker=None,
@@ -494,7 +501,7 @@ class PlotItemBuilder(object):
                            xaxis, yaxis)
     
     def histogram(self, data, bins=None, logscale=None,
-                  title=u"", color=None, xaxis="bottom", yaxis="left"):
+                  title="", color=None, xaxis="bottom", yaxis="left"):
         """
         Make 1D Histogram `plot item` 
         (:py:class:`guiqwt.histogram.HistogramItem` object)
@@ -551,32 +558,40 @@ class PlotItemBuilder(object):
             param.alpha = alpha
         interp_methods = {'nearest': 0, 'linear': 1, 'antialiasing': 5}
         param.interpolation = interp_methods[interpolation]
-        for key, val in kwargs.items():
+        for key, val in list(kwargs.items()):
             if val is not None:
                 setattr(param, key, val)
 
     def _get_image_data(self, data, filename, title, to_grayscale):
         if data is None:
             assert filename is not None
-            from guiqwt.io import imagefile_to_array
-            data = imagefile_to_array(filename, to_grayscale=to_grayscale)
+            from guiqwt import io
+            data = io.imread(filename, to_grayscale=to_grayscale)
         if title is None and filename is not None:
             title = osp.basename(filename)
         return data, filename, title
 
     @staticmethod
-    def compute_bounds(data, pixel_size):
+    def compute_bounds(data, pixel_size, center_on):
         """Return image bounds from *pixel_size* (scalar or tuple)"""
         if not isinstance(pixel_size, (tuple, list)):
             pixel_size = [pixel_size, pixel_size]
         dx, dy = pixel_size
         xmin, ymin = 0., 0.
         xmax, ymax = data.shape[1]*dx, data.shape[0]*dy
+        if center_on is not None:
+            xc, yc = center_on
+            dx, dy = .5*(xmax-xmin)-xc, .5*(ymax-ymin)-yc
+            xmin -= dx
+            xmax -= dx
+            ymin -= dy
+            ymax -= dy
         return xmin, xmax, ymin, ymax
         
     def image(self, data=None, filename=None, title=None, alpha_mask=None,
               alpha=None, background_color=None, colormap=None,
-              xdata=[None, None], ydata=[None, None], pixel_size=None,
+              xdata=[None, None], ydata=[None, None],
+              pixel_size=None, center_on=None,
               interpolation='linear', eliminate_outliers=None,
               xformat='%.1f', yformat='%.1f', zformat='%.1f'):
         """
@@ -594,10 +609,13 @@ class PlotItemBuilder(object):
                                  alpha_mask=alpha_mask, alpha=alpha)
         assert data.ndim == 2, "Data must have 2 dimensions"
         if pixel_size is None:
+            assert center_on is None, "Ambiguous parameters: both `center_on`"\
+                                      " and `xdata`/`ydata` were specified"
             xmin, xmax = xdata
             ymin, ymax = ydata
         else:
-            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size)
+            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size,
+                                                         center_on)
         self.__set_image_param(param, title, alpha_mask, alpha, interpolation,
                                background=background_color,
                                colormap=colormap,
@@ -613,7 +631,8 @@ class PlotItemBuilder(object):
 
     def maskedimage(self, data=None, mask=None, filename=None, title=None,
                     alpha_mask=False, alpha=1.0,
-                    xdata=[None, None], ydata=[None, None], pixel_size=None,
+                    xdata=[None, None], ydata=[None, None],
+                    pixel_size=None, center_on=None,
                     background_color=None, colormap=None,
                     show_mask=False, fill_value=None, interpolation='linear',
                     eliminate_outliers=None,
@@ -626,13 +645,16 @@ class PlotItemBuilder(object):
         assert isinstance(ydata, (tuple, list)) and len(ydata) == 2
         param = MaskedImageParam(title=_("Image"), icon='image.png')
         data, filename, title = self._get_image_data(data, filename, title,
-                                                     to_grayscale=False)
+                                                     to_grayscale=True)
         assert data.ndim == 2, "Data must have 2 dimensions"
         if pixel_size is None:
+            assert center_on is None, "Ambiguous parameters: both `center_on`"\
+                                      " and `xdata`/`ydata` were specified"
             xmin, xmax = xdata
             ymin, ymax = ydata
         else:
-            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size)
+            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size,
+                                                         center_on)
         self.__set_image_param(param, title, alpha_mask, alpha, interpolation,
                                background=background_color,
                                colormap=colormap,
@@ -649,7 +671,8 @@ class PlotItemBuilder(object):
 
     def rgbimage(self, data=None, filename=None, title=None,
                  alpha_mask=False, alpha=1.0,
-                 xdata=[None, None], ydata=[None, None], pixel_size=None,
+                 xdata=[None, None], ydata=[None, None],
+                 pixel_size=None, center_on=None,
                  interpolation='linear'):
         """
         Make a RGB image `plot item` from data
@@ -662,10 +685,13 @@ class PlotItemBuilder(object):
                                                      to_grayscale=False)
         assert data.ndim == 3, "RGB data must have 3 dimensions"
         if pixel_size is None:
+            assert center_on is None, "Ambiguous parameters: both `center_on`"\
+                                      " and `xdata`/`ydata` were specified"
             xmin, xmax = xdata
             ymin, ymax = ydata
         else:
-            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size)
+            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size,
+                                                         center_on)
         self.__set_image_param(param, title, alpha_mask, alpha, interpolation,
                                xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
         image = RGBImageItem(data, param)
@@ -1001,15 +1027,14 @@ class PlotItemBuilder(object):
         """
         return self.__shape(RectangleShape, x0, y0, x1, y1, title)
 
-    def ellipse(self, x0, y0, x1, y1, ratio, title=None):
+    def ellipse(self, x0, y0, x1, y1, title=None):
         """
         Make an ellipse shape `plot item` 
         (:py:class:`guiqwt.shapes.EllipseShape` object)
             * x0, y0, x1, y1: ellipse x-axis coordinates
-            * ratio: ratio between y-axis and x-axis lengths
             * title: label name (optional)
         """
-        shape = EllipseShape(x0, y0, x1, y1, ratio)
+        shape = EllipseShape(x0, y0, x1, y1)
         shape.set_style("plot", "shape/drag")
         if title is not None:
             shape.setTitle(title)
@@ -1022,7 +1047,7 @@ class PlotItemBuilder(object):
             * x0, y0, x1, y1: circle diameter coordinates
             * title: label name (optional)
         """
-        return self.ellipse(x0, y0, x1, y1, 1., title=title)
+        return self.ellipse(x0, y0, x1, y1, title=title)
 
     def segment(self, x0, y0, x1, y1, title=None):
         """
@@ -1091,7 +1116,7 @@ class PlotItemBuilder(object):
         return self.__annotated_shape(AnnotatedSegment,
                                       x0, y0, x1, y1, title, subtitle)
 
-    def info_label(self, anchor, comps, title=""):
+    def info_label(self, anchor, comps, title=None):
         """
         Make an info label `plot item` 
         (:py:class:`guiqwt.label.DataInfoLabel` object)
@@ -1099,7 +1124,7 @@ class PlotItemBuilder(object):
         basename = _("Computation")
         param = LabelParam(basename, icon='label.png')
         param.read_config(CONF, "plot", "info_label")
-        if title:
+        if title is not None:
             param.label = title
         else:
             global LABEL_COUNT
@@ -1111,6 +1136,28 @@ class PlotItemBuilder(object):
         c = ANCHOR_OFFSETS[anchor]
         param.xc, param.yc = c
         return DataInfoLabel(param, comps)
+    
+    def range_info_label(self, range, anchor, label,
+                         function=None, title=None):
+        """
+        Make an info label `plot item` showing an XRangeSelection object infos
+        
+        Default function is `lambda x, dx: (x, dx)`.
+
+        Example:
+        -------
+        
+        x = linspace(-10, 10, 10)
+        y = sin(sin(sin(x)))
+        range = make.range(-2, 2)
+        disp = make.range_info_label(range, 'BL', "x = %.1f ± %.1f cm",
+                                     lambda x, dx: (x, dx))
+        
+        (:py:class:`guiqwt.label.DataInfoLabel` object)
+        (see example: :py:mod:`guiqwt.tests.computations`)
+        """
+        info = RangeInfo(label, range, function)
+        return make.info_label(anchor, info, title=title)
 
     def computation(self, range, anchor, label, curve, function, title=None):
         """
