@@ -37,6 +37,8 @@ Reference
    :inherited-members:
 """
 
+from __future__ import print_function
+
 import weakref
 
 from guidata.qt.QtGui import (QVBoxLayout, QSizePolicy, QHBoxLayout, QToolBar,
@@ -54,7 +56,7 @@ from guiqwt.config import CONF, _
 from guiqwt.interfaces import ICSImageItemType, IPanel, IBasePlotItem
 from guiqwt.panels import PanelWidget, ID_XCS, ID_YCS, ID_OCS
 from guiqwt.curve import CurvePlot, ErrorBarCurveItem
-from guiqwt.image import ImagePlot, LUT_MAX
+from guiqwt.image import ImagePlot, LUT_MAX, get_image_from_qrect
 from guiqwt.styles import CurveParam
 from guiqwt.tools import ExportItemDataTool
 from guiqwt.geometry import translate, rotate, vector_norm, vector_angle
@@ -66,6 +68,7 @@ from guiqwt.signals import (SIG_MARKER_CHANGED, SIG_PLOT_LABELS_CHANGED,
                             SIG_MASK_CHANGED)
 from guiqwt.plot import PlotManager
 from guiqwt.builder import make
+from guiqwt.baseplot import canvas_to_axes, axes_to_canvas
 
 
 class CrossSectionItem(ErrorBarCurveItem):
@@ -132,24 +135,6 @@ class CrossSectionItem(ErrorBarCurveItem):
         raise NotImplementedError
 
 
-def get_image_data(plot, p0, p1, apply_lut=False):
-    """
-    Save rectangular plot area
-    p0, p1: resp. top left and bottom right points (QPoint objects)
-    """
-    from guiqwt.image import TrImageItem, get_image_from_plot, get_plot_qrect
-    items = plot.get_items(item_type=ICSImageItemType)
-    if not items:
-        raise TypeError, _("There is no supported image item in current plot.")
-    _src_x, _src_y, src_w, src_h = get_plot_qrect(plot, p0, p1).getRect()
-    trparams = [item.get_transform() for item in items
-                if isinstance(item, TrImageItem)]
-    if trparams:
-        src_w /= max([dx for _x, _y, _angle, dx, _dy, _hf, _vf in trparams])
-        src_h /= max([dy for _x, _y, _angle, _dx, dy, _hf, _vf in trparams])
-    return get_image_from_plot(plot, p0, p1, src_w, src_h, apply_lut=apply_lut)
-
-
 def get_rectangular_area(obj):
     """
     Return rectangular area covered by object
@@ -176,19 +161,21 @@ def get_plot_x_section(obj, apply_lut=False):
     plot = obj.plot()
     xmap = plot.canvasMap(plot.X_BOTTOM)
     xc0, xc1 = xmap.p1(), xmap.p2()
-    _xc0, yc0 = obj.axes_to_canvas(0, y0)
+    _xc0, yc0 = axes_to_canvas(obj, 0, y0)
     if plot.get_axis_direction("left"):
         yc1 = yc0+1
     else:
         yc1 = yc0-1
     try:
-        data = get_image_data(plot, QPoint(xc0, yc0), QPoint(xc1, yc1),
-                              apply_lut=apply_lut)
+        #TODO: eventually add an option to apply interpolation algorithm
+        data = get_image_from_qrect(plot, QPoint(xc0, yc0), QPoint(xc1, yc1),
+                                    apply_lut=apply_lut,
+                                    apply_interpolation=False)
     except (ValueError, ZeroDivisionError, TypeError):
         return np.array([]), np.array([])
     y = data.mean(axis=0)
-    x0, _y0 = obj.canvas_to_axes(QPoint(xc0, yc0))
-    x1, _y1 = obj.canvas_to_axes(QPoint(xc1, yc1))
+    x0, _y0 = canvas_to_axes(obj, QPoint(xc0, yc0))
+    x1, _y1 = canvas_to_axes(obj, QPoint(xc1, yc1))
     x = np.linspace(x0, x1, len(y))
     return x, y
 
@@ -203,16 +190,17 @@ def get_plot_y_section(obj, apply_lut=False):
     yc0, yc1 = ymap.p1(), ymap.p2()
     if plot.get_axis_direction("left"):
         yc1, yc0 = yc0, yc1
-    xc0, _yc0 = obj.axes_to_canvas(x0, 0)
+    xc0, _yc0 = axes_to_canvas(obj, x0, 0)
     xc1 = xc0+1
     try:
-        data = get_image_data(plot, QPoint(xc0, yc0), QPoint(xc1, yc1),
-                              apply_lut=apply_lut)
+        data = get_image_from_qrect(plot, QPoint(xc0, yc0), QPoint(xc1, yc1),
+                                    apply_lut=apply_lut,
+                                    apply_interpolation=False)
     except (ValueError, ZeroDivisionError, TypeError):
         return np.array([]), np.array([])
     y = data.mean(axis=1)
-    _x0, y0 = obj.canvas_to_axes(QPoint(xc0, yc0))
-    _x1, y1 = obj.canvas_to_axes(QPoint(xc1, yc1))
+    _x0, y0 = canvas_to_axes(obj, QPoint(xc0, yc0))
+    _x1, y1 = canvas_to_axes(obj, QPoint(xc1, yc1))
     x = np.linspace(y0, y1, len(y))
     return x, y
 
@@ -224,8 +212,8 @@ def get_plot_average_x_section(obj, apply_lut=False):
     (RectangleShape, AnnotatedRectangle, etc.)
     """
     x0, y0, x1, y1 = obj.get_rect()
-    xc0, yc0 = obj.axes_to_canvas(x0, y0)
-    xc1, yc1 = obj.axes_to_canvas(x1, y1)
+    xc0, yc0 = axes_to_canvas(obj, x0, y0)
+    xc1, yc1 = axes_to_canvas(obj, x1, y1)
     invert = False
     if xc0 > xc1:
         invert = True
@@ -234,8 +222,10 @@ def get_plot_average_x_section(obj, apply_lut=False):
     if (ydir and yc0 > yc1) or (not ydir and yc0 < yc1):
         yc1, yc0 = yc0, yc1
     try:
-        data = get_image_data(obj.plot(), QPoint(xc0, yc0), QPoint(xc1, yc1),
-                              apply_lut=apply_lut)
+        data = get_image_from_qrect(obj.plot(),
+                                    QPoint(xc0, yc0), QPoint(xc1, yc1),
+                                    apply_lut=apply_lut,
+                                    apply_interpolation=False)
     except (ValueError, ZeroDivisionError, TypeError):
         return np.array([]), np.array([])
     y = data.mean(axis=0)
@@ -251,8 +241,8 @@ def get_plot_average_y_section(obj, apply_lut=False):
     (RectangleShape, AnnotatedRectangle, etc.)
     """
     x0, y0, x1, y1 = obj.get_rect()
-    xc0, yc0 = obj.axes_to_canvas(x0, y0)
-    xc1, yc1 = obj.axes_to_canvas(x1, y1)
+    xc0, yc0 = axes_to_canvas(obj, x0, y0)
+    xc1, yc1 = axes_to_canvas(obj, x1, y1)
     invert = False
     ydir = obj.plot().get_axis_direction("left")
     if (ydir and yc0 > yc1) or (not ydir and yc0 < yc1):
@@ -261,8 +251,10 @@ def get_plot_average_y_section(obj, apply_lut=False):
     if xc0 > xc1:
         xc1, xc0 = xc0, xc1
     try:
-        data = get_image_data(obj.plot(), QPoint(xc0, yc0), QPoint(xc1, yc1),
-                              apply_lut=apply_lut)
+        data = get_image_from_qrect(obj.plot(),
+                                    QPoint(xc0, yc0), QPoint(xc1, yc1),
+                                    apply_lut=apply_lut,
+                                    apply_interpolation=False)
     except (ValueError, ZeroDivisionError, TypeError):
         return np.array([]), np.array([])
     y = data.mean(axis=1)
@@ -367,7 +359,7 @@ class CrossSectionPlot(CurvePlot):
         elif self._width is not None:
             self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)
             
-        self.label = make.label(self.LABEL_TEXT, "C", (0,0), "C")
+        self.label = make.label(self.LABEL_TEXT, "C", (0, 0), "C")
         self.label.set_readonly(True)
         self.add_item(self.label)
         
@@ -426,7 +418,7 @@ class CrossSectionPlot(CurvePlot):
         self.known_items[source] = curve
         
     def get_cross_section_curves(self):
-        return self.known_items.values()
+        return list(self.known_items.values())
 
     def items_changed(self, plot):
         # Del obsolete cross section items
@@ -465,7 +457,7 @@ class CrossSectionPlot(CurvePlot):
         self.update_plot(marker)
 
     def is_shape_known(self, shape):
-        for shapes in self._shapes.values():
+        for shapes in list(self._shapes.values()):
             if shape in shapes:
                 return True
         else:
@@ -501,7 +493,7 @@ class CrossSectionPlot(CurvePlot):
             return
         if self.label.isVisible():
             self.label.hide()
-        for index, (_item, curve) in enumerate(self.known_items.iteritems()):
+        for index, (_item, curve) in enumerate(iter(list(self.known_items.items()))):
             if not self.perimage_mode and index > 0:
                 curve.hide()
             else:
@@ -881,7 +873,7 @@ def compute_oblique_section(item, obj):
     try:
         xdata -= xdata[0]
     except IndexError:
-        print xdata, ydata
+        print(xdata, ydata)
     return xdata, ydata
 
 # Oblique cross section item
