@@ -100,20 +100,21 @@ Reference
    :members:
 """
 
-from __future__ import with_statement
+from __future__ import with_statement, print_function
 
-import sys
 import warnings
 import numpy as np
 
 from guidata.qt.QtGui import (QMenu, QListWidget, QListWidgetItem, QVBoxLayout,
-                              QToolBar, QMessageBox, QBrush, QColor, QPen, QPolygonF)
+                              QToolBar, QMessageBox, QBrush, QColor, QPen,
+                              QPolygonF)
 from guidata.qt.QtCore import (Qt, QPoint, QPointF, QLineF, SIGNAL, QRectF,
                                QLine)
 
 from guidata.utils import assert_interfaces_valid, update_dataset
 from guidata.configtools import get_icon, get_image_layout
 from guidata.qthelpers import create_action, add_actions
+from guidata.py3compat import is_text_string, maxsize
 
 # Local imports
 from guiqwt.transitional import (QwtPlotCurve, QwtPlotGrid, QwtPlotItem,
@@ -123,25 +124,24 @@ from guiqwt.interfaces import (IBasePlotItem, IDecoratorItemType,
                                ISerializableType, ICurveItemType,
                                ITrackableItemType, IPanel)
 from guiqwt.panels import PanelWidget, ID_ITEMLIST
-from guiqwt.baseplot import BasePlot
+from guiqwt.baseplot import BasePlot, canvas_to_axes
 from guiqwt.styles import GridParam, CurveParam, ErrorBarParam, SymbolParam
 from guiqwt.shapes import Marker
 from guiqwt.signals import (SIG_ACTIVE_ITEM_CHANGED, SIG_ITEMS_CHANGED,
-                            SIG_ITEM_REMOVED, SIG_AXIS_DIRECTION_CHANGED,
-                            SIG_PLOT_AXIS_CHANGED)
+                            SIG_AXIS_DIRECTION_CHANGED, SIG_PLOT_AXIS_CHANGED)
 
 def _simplify_poly(pts, off, scale, bounds):
     ax, bx, ay, by = scale
     xm, ym, xM, yM = bounds
-    a = np.array( [[ax,ay]] )
-    b = np.array( [[bx,by]] )
+    a = np.array( [[ax, ay]] )
+    b = np.array( [[bx, by]] )
     _pts = a*pts+b
     poly = []
     NP = off.shape[0]
-    for i in xrange(off.shape[0]):
-        i0 = off[i,1]
+    for i in range(off.shape[0]):
+        i0 = off[i, 1]
         if i+1<NP:
-            i1 = off[i+1,1]
+            i1 = off[i+1, 1]
         else:
             i1 = pts.shape[0]
         poly.append( (_pts[i0:i1], i) )
@@ -173,9 +173,9 @@ def seg_dist(P, P0, P1):
             return abs(u*np.sin(angle))
 
 def test_seg_dist():
-    print seg_dist(QPointF(200, 100), QPointF(150, 196), QPointF(250, 180))
-    print seg_dist(QPointF(200, 100), QPointF(190, 196), QPointF(210, 180))
-    print seg_dist(QPointF(201, 105), QPointF(201, 196), QPointF(201, 180))
+    print(seg_dist(QPointF(200, 100), QPointF(150, 196), QPointF(250, 180)))
+    print(seg_dist(QPointF(200, 100), QPointF(190, 196), QPointF(210, 180)))
+    print(seg_dist(QPointF(201, 105), QPointF(201, 196), QPointF(201, 180)))
 
 def norm2(v):
     return (v**2).sum(axis=1)
@@ -191,7 +191,7 @@ def seg_dist_v(P, X0, Y0, X1, Y1):
     dP = np.array(P).reshape(1, 2) - PP
     nV = np.sqrt(norm2(V)).clip(1e-12) # clip: avoid division by zero
     w2 = V/nV[:, np.newaxis]
-    w = np.array([ -w2[:,1], w2[:,0] ]).T
+    w = np.array([ -w2[:, 1], w2[:, 0] ]).T
     distances = np.fabs((dP*w).sum(axis=1))
     ix = distances.argmin()
     return ix, distances[ix]
@@ -201,7 +201,7 @@ def test_seg_dist_v():
     a=(np.arange(10.)**2).reshape(5, 2)
     ix, dist = seg_dist_v((2.1, 3.3), a[:-1, 0], a[:-1, 1],
                           a[1:, 0], a[1:, 1])
-    print ix, dist
+    print(ix, dist)
     assert ix == 0
 
 if __name__ == "__main__":
@@ -240,7 +240,7 @@ class GridItem(QwtPlotGrid):
     
     def attach(self, plot):
         """Reimplemented to update plot canvas background"""
-        QwtPlotGrid.attach(self,plot)
+        QwtPlotGrid.attach(self, plot)
         self.update_params()
 
     def set_readonly(self, state):
@@ -294,7 +294,7 @@ class GridItem(QwtPlotGrid):
         pass
 
     def hit_test(self, pos):
-        return sys.maxint, 0, False, None
+        return maxsize, 0, False, None
 
     def move_local_point_to(self, handle, pos, ctrl=None):
         pass
@@ -323,7 +323,7 @@ class CurveItem(QwtPlotCurve):
     Construct a curve `plot item` with the parameters *curveparam*
     (see :py:class:`guiqwt.styles.CurveParam`)
     """
-    __implements__ = (IBasePlotItem,)
+    __implements__ = (IBasePlotItem, ISerializableType)
     
     _readonly = False
     _private = False
@@ -402,6 +402,24 @@ class CurveItem(QwtPlotCurve):
         self.setZ(z)
         self.update_params()
 
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        writer.write(self._x, group_name='Xdata')
+        writer.write(self._y, group_name='Ydata')
+        writer.write(self.z(), group_name='z')
+        self.curveparam.update_param(self)
+        writer.write(self.curveparam, group_name='curveparam')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        self.curveparam = CurveParam(_("Curve"), icon='curve.png')
+        reader.read('curveparam', instance=self.curveparam)
+        x = reader.read(group_name='Xdata', func=reader.read_array)
+        y = reader.read(group_name='Ydata', func=reader.read_array)
+        self.set_data(x, y)
+        self.setZ(reader.read('z'))
+        self.update_params()
+    
     def set_readonly(self, state):
         """Set object readonly state"""
         self._readonly = state
@@ -458,7 +476,7 @@ class CurveItem(QwtPlotCurve):
         """Calcul de la distance d'un point à une courbe
         renvoie (dist, handle, inside)"""
         if self.is_empty():
-            return sys.maxint, 0, False, None
+            return maxsize, 0, False, None
         plot = self.plot()
         ax = self.xAxis()
         ay = self.yAxis()
@@ -466,8 +484,14 @@ class CurveItem(QwtPlotCurve):
         py = plot.invTransform(ay, pos.y())
         # On cherche les 4 points qui sont les plus proches en X et en Y
         # avant et après ie tels que p1x < x < p2x et p3y < y < p4y
-        dx = 1/(self._x-px)
-        dy = 1/(self._y-py)
+        tmpx = self._x - px
+        tmpy = self._y - py
+        if np.count_nonzero(tmpx) != len(tmpx) or\
+           np.count_nonzero(tmpy) != len(tmpy):
+            # Avoid dividing by zero warning when computing dx or dy
+            return maxsize, 0, False, None
+        dx = 1/tmpx
+        dy = 1/tmpy
         i0 = dx.argmin()
         i1 = dx.argmax()
         i2 = dy.argmin()
@@ -516,18 +540,12 @@ class CurveItem(QwtPlotCurve):
                 return self._x[i-1], self._y[i-1]
         return self._x[i], self._y[i]
 
-    def canvas_to_axes(self, pos):
-        plot = self.plot()
-        ax = self.xAxis()
-        ay = self.yAxis()
-        return plot.invTransform(ax, pos.x()), plot.invTransform(ay, pos.y())
-
     def move_local_point_to(self, handle, pos, ctrl=None):
         if self.immutable:
             return
         if handle < 0 or handle > self._x.shape[0]:
             return
-        x, y = self.canvas_to_axes(pos)
+        x, y = canvas_to_axes(self, pos)
         self._x[handle] = x
         self._y[handle] = y
         self.setData(self._x, self._y)
@@ -536,8 +554,8 @@ class CurveItem(QwtPlotCurve):
     def move_local_shape(self, old_pos, new_pos):
         """Translate the shape such that old_pos becomes new_pos
         in canvas coordinates"""
-        nx, ny = self.canvas_to_axes(new_pos)
-        ox, oy = self.canvas_to_axes(old_pos)
+        nx, ny = canvas_to_axes(self, new_pos)
+        ox, oy = canvas_to_axes(self, old_pos)
         self._x += (nx-ox)
         self._y += (ny-oy)
         self.setData(self._x, self._y)
@@ -551,7 +569,7 @@ class CurveItem(QwtPlotCurve):
         self._y += delta_y
         self.setData(self._x, self._y)
 
-    def update_params(self):
+    def update_params(self):        
         self.curveparam.update_curve(self)
         if self.selected:
             self.select()
@@ -572,7 +590,7 @@ class PolygonMapItem(QwtPlotItem):
     Construct a curve `plot item` with the parameters *curveparam*
     (see :py:class:`guiqwt.styles.CurveParam`)
     """
-    __implements__ = (IBasePlotItem,)
+    __implements__ = (IBasePlotItem, ISerializableType)
     
     _readonly = False
     _private = False
@@ -621,18 +639,19 @@ class PolygonMapItem(QwtPlotItem):
 
     def setPen(self, x):
         pass
-    def setBrush(self,x):
+    def setBrush(self, x):
         pass
-    def setSymbol(self,x):
+    def setSymbol(self, x):
         pass
-    def setCurveAttribute(self,x,y):
+    def setCurveAttribute(self, x, y):
         pass
-    def setStyle(self,x):
+    def setStyle(self, x):
         pass
-    def setCurveType(self,x):
+    def setCurveType(self, x):
         pass
-    def setBaseline(self,x):
+    def setBaseline(self, x):
         pass
+
     def __reduce__(self):
         state = (self.curveparam, self._pts, self._n, self._c, self.z())
         res = ( PolygonMapItem, (), state )
@@ -645,6 +664,26 @@ class PolygonMapItem(QwtPlotItem):
         self.setZ(z)
         self.update_params()
 
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        writer.write(self._pts, group_name='Pdata')
+        writer.write(self._n, group_name='Ndata')
+        writer.write(self._c, group_name='Cdata')
+        writer.write(self.z(), group_name='z')
+        self.curveparam.update_param(self)
+        writer.write(self.curveparam, group_name='curveparam')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        pts = reader.read(group_name='Pdata', func=reader.read_array)
+        n = reader.read(group_name='Ndata', func=reader.read_array)
+        c = reader.read(group_name='Cdata', func=reader.read_array)
+        self.set_data(pts, n, c)
+        self.setZ(reader.read('z'))
+        self.curveparam = CurveParam(_("PolygonMap"), icon='curve.png')
+        reader.read('curveparam', instance=self.curveparam)
+        self.update_params()
+
     def set_readonly(self, state):
         """Set object readonly state"""
         self._readonly = state
@@ -694,7 +733,7 @@ class PolygonMapItem(QwtPlotItem):
         self._c = np.array(c, copy=False)
         xmin, ymin = self._pts.min(axis=0)
         xmax, ymax = self._pts.max(axis=0)
-        self.bounds = QRectF(xmin,ymin,xmax-xmin,ymax-ymin)
+        self.bounds = QRectF(xmin, ymin, xmax-xmin, ymax-ymin)
         
     def is_empty(self):
         """Return True if item data is empty"""
@@ -704,7 +743,7 @@ class PolygonMapItem(QwtPlotItem):
         """Calcul de la distance d'un point à une courbe
         renvoie (dist, handle, inside)"""
         if self.is_empty():
-            return sys.maxint, 0, False, None
+            return maxsize, 0, False, None
         plot = self.plot()
         # TODO
         return distance, i, False, None
@@ -743,7 +782,7 @@ class PolygonMapItem(QwtPlotItem):
         self.update_params()
 
     def draw(self, painter, xMap, yMap, canvasRect):
-        from time import time
+        #from time import time
         p1x = xMap.p1()
         s1x = xMap.s1()
         ax = (xMap.p2() - p1x)/(xMap.s2()-s1x)
@@ -763,11 +802,11 @@ class PolygonMapItem(QwtPlotItem):
         #t2 = time()
         for poly, num in polygons:
             points = []
-            for i in xrange(poly.shape[0]):
-                points.append(QPointF(poly[i,0],poly[i,1]))
+            for i in range(poly.shape[0]):
+                points.append(QPointF(poly[i, 0], poly[i, 1]))
             pg = QPolygonF(points)
-            fgcol.setRgba(int(_c[num,0]))
-            bgcol.setRgba(int(_c[num,1]))
+            fgcol.setRgba(int(_c[num, 0]))
+            bgcol.setRgba(int(_c[num, 1]))
             painter.setPen(QPen(fgcol))
             painter.setBrush(QBrush(bgcol))
             painter.drawPolygon(pg)
@@ -806,6 +845,29 @@ class ErrorBarCurveItem(CurveItem):
         self._dx = None
         self._dy = None
         self._minmaxarrays = {}
+
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        super(ErrorBarCurveItem, self).serialize(writer)
+        writer.write(self._dx, group_name='dXdata')
+        writer.write(self._dy, group_name='dYdata')
+        self.errorbarparam.update_param(self)
+        writer.write(self.errorbarparam, group_name='errorbarparam')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        self.curveparam = CurveParam(_("Curve"), icon='curve.png')
+        reader.read('curveparam', instance=self.curveparam)
+        self.errorbarparam = ErrorBarParam(_("Error bars"),
+                                           icon='errorbar.png')
+        reader.read('errorbarparam', instance=self.errorbarparam)
+        x = reader.read(group_name='Xdata', func=reader.read_array)
+        y = reader.read(group_name='Ydata', func=reader.read_array)
+        dx = reader.read(group_name='dXdata', func=reader.read_array)
+        dy = reader.read(group_name='dYdata', func=reader.read_array)
+        self.set_data(x, y, dx, dy)
+        self.setZ(reader.read('z'))
+        self.update_params()
         
     def unselect(self):
         """Unselect item"""
@@ -924,7 +986,7 @@ class ErrorBarCurveItem(CurveItem):
         x, y, xmin, xmax, ymin, ymax = self.get_minmax_arrays(all_values=False)
         tx = vmap(xMap, x)
         ty = vmap(yMap, y)
-        RN = xrange(len(tx))
+        RN = list(range(len(tx)))
         if self.errorOnTop:
             QwtPlotCurve.draw(self, painter, xMap, yMap, canvasRect)
         
@@ -1069,7 +1131,7 @@ class ItemListWidget(QListWidget):
     def __is_selection_contiguous(self):
         indexes = sorted([self.row(lw_item) for lw_item
                           in self.selectedItems()])
-        return len(indexes) <= 1 or range(indexes[0], indexes[-1]+1) == indexes
+        return len(indexes) <= 1 or list(range(indexes[0], indexes[-1]+1)) == indexes
         
     def get_selected_items(self):
         """Return selected QwtPlot items
@@ -1212,8 +1274,6 @@ class ItemListWidget(QListWidget):
             items = self.get_selected_items()
             self.plot.del_items(items)
             self.plot.replot()
-            for item in items:
-                self.plot.emit(SIG_ITEM_REMOVED, item)
         
 
 class PlotItemList(PanelWidget):
@@ -1268,7 +1328,8 @@ class CurvePlot(BasePlot):
         * axes_synchronised: keep all x and y axes synchronised when zomming or
                              panning
     """
-    AUTOSCALE_TYPES = (CurveItem,PolygonMapItem)
+    DEFAULT_ITEM_TYPE = ICurveItemType
+    AUTOSCALE_TYPES = (CurveItem, PolygonMapItem)
     def __init__(self, parent=None, title=None, xlabel=None, ylabel=None,
                  xunit=None, yunit=None, gridparam=None,
                  section="plot", axes_synchronised=False):
@@ -1480,6 +1541,16 @@ class CurvePlot(BasePlot):
             self.setAxisScale(k, o1, o2)
         self.replot()
 
+    def get_default_item(self):
+        """Return default item, depending on plot's default item type
+        (e.g. for a curve plot, this is a curve item type).
+        
+        Return nothing if there is more than one item matching 
+        the default item type."""
+        items = self.get_items(item_type=self.DEFAULT_ITEM_TYPE)
+        if len(items) == 1:
+            return items[0]
+
     #---- BasePlot API ---------------------------------------------------------
     def add_item(self, item, z=None):
         """
@@ -1618,25 +1689,25 @@ class CurvePlot(BasePlot):
         if title is not None:
             self.set_title(title)
         if xlabel is not None:
-            if isinstance(xlabel, basestring):
+            if is_text_string(xlabel):
                 xlabel = (xlabel, "")
             for label, axis in zip(xlabel, ("bottom", "top")):
                 if label is not None:
                     self.set_axis_title(axis, label)
         if ylabel is not None:
-            if isinstance(ylabel, basestring):
+            if is_text_string(ylabel):
                 ylabel = (ylabel, "")
             for label, axis in zip(ylabel, ("left", "right")):
                 if label is not None:
                     self.set_axis_title(axis, label)
         if xunit is not None:
-            if isinstance(xunit, basestring):
+            if is_text_string(xunit):
                 xunit = (xunit, "")
             for unit, axis in zip(xunit, ("bottom", "top")):
                 if unit is not None:
                     self.set_axis_unit(axis, unit)
         if yunit is not None:
-            if isinstance(yunit, basestring):
+            if is_text_string(yunit):
                 yunit = (yunit, "")
             for unit, axis in zip(yunit, ("left", "right")):
                 if unit is not None:
