@@ -43,6 +43,8 @@ Reference
    :members:
    :inherited-members:
 """
+
+import weakref
 import numpy as np
 from guidata.qt.QtCore import Qt
 from guidata.qt.QtGui import QHBoxLayout, QVBoxLayout, QToolBar
@@ -66,7 +68,8 @@ from guiqwt.shapes import XRangeSelection
 from guiqwt.tools import (SelectTool, BasePlotMenuTool, SelectPointTool,
                           AntiAliasingTool)
 from guiqwt.signals import (SIG_RANGE_CHANGED, SIG_VOI_CHANGED,
-                            SIG_ITEM_SELECTION_CHANGED, SIG_ACTIVE_ITEM_CHANGED)
+                            SIG_ITEM_SELECTION_CHANGED, SIG_ITEM_REMOVED,
+                            SIG_ACTIVE_ITEM_CHANGED)
 from guiqwt.plot import PlotManager
 
 
@@ -122,13 +125,18 @@ class HistogramItem(CurveItem):
         self.setCurveAttribute(QwtPlotCurve.Inverted)
             
     def set_hist_source(self, src):
-        """
-        Set histogram source
+        """Set histogram source
         (source: object with method 'get_histogram',
-         e.g. objects derived from guiqwt.image.ImageItem)
-        """
-        self.source = src
+         e.g. objects derived from guiqwt.image.ImageItem)"""
+        self.source = weakref.ref(src)
         self.update_histogram()
+
+    def get_hist_source(self):
+        """Return histogram source
+        (source: object with method 'get_histogram',
+         e.g. objects derived from guiqwt.image.ImageItem)"""
+        if self.source is not None:
+            return self.source()
         
     def set_hist_data(self, data):
         """Set histogram data"""
@@ -152,10 +160,10 @@ class HistogramItem(CurveItem):
         return self.bins
         
     def compute_histogram(self):
-        return self.source.get_histogram(self.bins)
+        return self.get_hist_source().get_histogram(self.bins)
         
     def update_histogram(self):
-        if self.source is None:
+        if self.get_hist_source() is None:
             return
         hist, bin_edges = self.compute_histogram()
         hist = np.concatenate((hist, [0]))
@@ -230,16 +238,17 @@ class LevelsHistogram(CurvePlot):
             return
         self.connect(self, SIG_VOI_CHANGED, plot.notify_colormap_changed)
         self.connect(plot, SIG_ITEM_SELECTION_CHANGED, self.selection_changed)
+        self.connect(plot, SIG_ITEM_REMOVED, self.item_removed)
         self.connect(plot, SIG_ACTIVE_ITEM_CHANGED, self.active_item_changed)
 
     def tracked_items_gen(self):
-        for plot, items in self._tracked_items.items():
-            for item in items.items():
+        for plot, items in list(self._tracked_items.items()):
+            for item in list(items.items()):
                 yield item # tuple item,curve
 
     def __del_known_items(self, known_items, items):
         del_curves = []
-        for item in known_items.keys():
+        for item in list(known_items.keys()):
             if item not in items:
                 curve = known_items.pop(item)
                 del_curves.append(curve)
@@ -253,7 +262,7 @@ class LevelsHistogram(CurvePlot):
             self.__del_known_items(known_items, items)
             if len(items) == 1:
                 # Removing any cached item for other plots
-                for other_plot, _items in self._tracked_items.items():
+                for other_plot, _items in list(self._tracked_items.items()):
                     if other_plot is not plot:
                         if not other_plot.get_selected_items(
                                                 item_type=IVoiImageItemType):
@@ -262,7 +271,7 @@ class LevelsHistogram(CurvePlot):
         else:
             # if all items are deselected we keep the last known
             # selection (for one plot only)
-            for other_plot, _items in self._tracked_items.items():
+            for other_plot, _items in list(self._tracked_items.items()):
                 if other_plot.get_selected_items(item_type=IVoiImageItemType):
                     self.__del_known_items(known_items, [])
                     break
@@ -298,6 +307,12 @@ class LevelsHistogram(CurvePlot):
             self.set_axis_limits("left", ymin, ymax)
             self.replot()
 
+    def item_removed(self, item):
+        for plot, items in list(self._tracked_items.items()):
+            if item in items:
+                items.pop(item)
+                break
+
     def active_item_changed(self, plot):
         items = plot.get_selected_items(item_type=IVoiImageItemType)
         if not items:
@@ -460,13 +475,13 @@ class ContrastAdjustment(PanelWidget):
         """Configure panel"""
         self.min_select_tool = self.manager.add_tool(SelectPointTool,
                                        title=_("Minimum level"),
-                                       on_active_item=True,mode="create",
+                                       on_active_item=True, mode="create",
                                        tip=_("Select minimum level on image"),
                                        toolbar_id="contrast",
                                        end_callback=self.apply_min_selection)
         self.max_select_tool = self.manager.add_tool(SelectPointTool,
                                        title=_("Maximum level"),
-                                       on_active_item=True,mode="create",
+                                       on_active_item=True, mode="create",
                                        tip=_("Select maximum level on image"),
                                        toolbar_id="contrast",
                                        end_callback=self.apply_max_selection)        
@@ -490,7 +505,7 @@ class ContrastAdjustment(PanelWidget):
                                      tip=_("Eliminate levels histogram "
                                            "outliers and scale the image's "
                                            "display range accordingly") )
-        add_actions(self.toolbar,[fullrange_ac, autorange_ac])
+        add_actions(self.toolbar, [fullrange_ac, autorange_ac])
     
     def eliminate_outliers(self):
         def apply(param):
