@@ -50,25 +50,27 @@ from guidata.qt.QtGui import QPen, QColor, QTextDocument
 from guidata.qt.QtCore import QRectF
 
 from guidata.utils import assert_interfaces_valid, update_dataset
+from guidata.py3compat import to_text_string
 
 # Local imports
 from guiqwt.transitional import QwtPlotItem
-from guiqwt.config import CONF
+from guiqwt.config import CONF, _
 from guiqwt.curve import CurveItem
-from guiqwt.interfaces import IBasePlotItem, IShapeItemType
+from guiqwt.interfaces import IBasePlotItem, IShapeItemType, ISerializableType
+from guiqwt.styles import LabelParam
 from guiqwt.signals import SIG_ITEM_MOVED
 
 
 ANCHORS = {
-           "TL" : lambda r: (r.left(),r.top()),
-           "TR" : lambda r: (r.right(),r.top()),
-           "BL" : lambda r: (r.left(),r.bottom()),
-           "BR" : lambda r: (r.right(),r.bottom()),
-           "L"  : lambda r: (r.left(), (r.top()+r.bottom())/2.),
-           "R"  : lambda r: (r.right(), (r.top()+r.bottom())/2.),
-           "T"  : lambda r: ((r.left()+r.right())/2.0, r.top()),
-           "B"  : lambda r: ((r.left()+r.right())/2.0, r.bottom()),
-           "C"  : lambda r: ((r.left()+r.right())/2.0, (r.top()+r.bottom())/2.),
+           "TL": lambda r: (r.left(), r.top()),
+           "TR": lambda r: (r.right(), r.top()),
+           "BL": lambda r: (r.left(), r.bottom()),
+           "BR": lambda r: (r.right(), r.bottom()),
+           "L": lambda r: (r.left(), (r.top()+r.bottom())/2.),
+           "R": lambda r: (r.right(), (r.top()+r.bottom())/2.),
+           "T": lambda r: ((r.left()+r.right())/2.0, r.top()),
+           "B": lambda r: ((r.left()+r.right())/2.0, r.bottom()),
+           "C": lambda r: ((r.left()+r.right())/2.0, (r.top()+r.bottom())/2.),
            }
 
 
@@ -82,7 +84,7 @@ class AbstractLabelItem(QwtPlotItem):
     _readonly = False
     _private = False
     
-    def __init__(self, labelparam):
+    def __init__(self, labelparam=None):
         super(AbstractLabelItem, self).__init__()
         self.selected = False
         self.anchor = None
@@ -90,25 +92,35 @@ class AbstractLabelItem(QwtPlotItem):
         self.C = None
         self.border_pen = None
         self.bg_brush = None
-        self.labelparam = labelparam
-        self.labelparam.update_label(self)
+        if labelparam is None:
+            self.labelparam = LabelParam(_("Label"), icon='label.png')
+        else:
+            self.labelparam = labelparam
+            self.labelparam.update_label(self)
 
     def set_style(self, section, option):
         self.labelparam.read_config(CONF, section, option)
         self.labelparam.update_label(self)
 
-    def get_state(self):
-        return (self.labelparam,)
-
-    def __setstate__(self, state):
-        self.labelparam = state[0]
+    def __reduce__(self):
+        return (self.__class__, (self.labelparam,))
+    
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        self.labelparam.update_param(self)
+        writer.write(self.labelparam, group_name='labelparam')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        self.labelparam = LabelParam(_("Label"), icon='label.png')
+        reader.read('labelparam', instance=self.labelparam)
         self.labelparam.update_label(self)
         
     def get_text_rect(self):
-        return QRectF(0.0, 0.0, 10.,10.)
+        return QRectF(0.0, 0.0, 10., 10.)
 
     def types(self):
-        return (IShapeItemType,)
+        return (IShapeItemType, )
     
     def set_text_style(self, font=None, color=None):
         raise NotImplementedError
@@ -203,7 +215,7 @@ class AbstractLabelItem(QwtPlotItem):
         canvasRect = plot.canvas().contentsRect()
         xMap = plot.canvasMap(self.xAxis())
         yMap = plot.canvasMap(self.yAxis())
-        x,y = self.get_top_left(xMap, yMap, canvasRect)
+        x, y = self.get_top_left(xMap, yMap, canvasRect)
         rct = QRectF(x, y, rect.width(), rect.height())
         inside = rct.contains( pos.x(), pos.y())
         if inside:
@@ -278,22 +290,35 @@ class AbstractLabelItem(QwtPlotItem):
         
 
 class LabelItem(AbstractLabelItem):
-    __implements__ = (IBasePlotItem,)
+    __implements__ = (IBasePlotItem, ISerializableType)
 
-    def __init__(self, text, dataset):
-        self.text_string = text
+    def __init__(self, text=None, labelparam=None):
+        self.text_string = '' if text is None else text
         self.text = QTextDocument()
-        super(LabelItem, self).__init__(dataset)
+        super(LabelItem, self).__init__(labelparam)
     
     def __reduce__(self):
         return (self.__class__, (self.text_string, self.labelparam))
+    
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        super(LabelItem, self).serialize(writer)
+        writer.write(self.text_string, group_name='text')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        super(LabelItem, self).deserialize(reader)
+        self.set_text(reader.read('text', func=reader.read_unicode))
+
+    def types(self):
+        return (IShapeItemType, ISerializableType)
 
     def set_pos(self, x, y):
         self.G = x, y
         self.labelparam.xg, self.labelparam.yg = x, y
         
     def get_plain_text(self):
-        return unicode(self.text.toPlainText())
+        return to_text_string(self.text.toPlainText())
         
     def set_text(self, text=None):
         if text is not None:
@@ -335,17 +360,17 @@ LEGEND_SPACEH = 5  # Spacing between border, sample, text, border
 LEGEND_SPACEV = 3  # Vertical space between items
 
 class LegendBoxItem(AbstractLabelItem):
-    __implements__ = (IBasePlotItem,)
+    __implements__ = (IBasePlotItem, ISerializableType)
 
-    def __init__(self, dataset):
+    def __init__(self, labelparam=None):
         self.font = None
         self.color = None
-        super(LegendBoxItem, self).__init__(dataset)
+        super(LegendBoxItem, self).__init__(labelparam)
         # saves the last computed sizes
         self.sizes = 0.0, 0.0, 0.0, 0.0
 
-    def __reduce__(self):
-        return (self.__class__, (self.labelparam,) )
+    def types(self):
+        return (IShapeItemType, ISerializableType)
 
     def get_legend_items(self):
         plot = self.plot()
@@ -407,7 +432,7 @@ class LegendBoxItem(AbstractLabelItem):
             painter.setBrush(ibrush)
             painter.drawLine( x0, y0+height/2, x0+LEGEND_WIDTH, y0+height/2)
             x1 = x0+LEGEND_SPACEH+LEGEND_WIDTH
-            painter.translate(x1,y0)
+            painter.translate(x1, y0)
             text.drawContents(painter)
             painter.restore()
             y0 += height+LEGEND_SPACEV
@@ -440,9 +465,9 @@ class LegendBoxItem(AbstractLabelItem):
 assert_interfaces_valid(LegendBoxItem)
 
 class SelectedLegendBoxItem(LegendBoxItem):
-    def __init__(self, dataset, itemlist):
+    def __init__(self, dataset=None, itemlist=None):
         super(SelectedLegendBoxItem, self).__init__(dataset)
-        self.itemlist = itemlist
+        self.itemlist = [] if itemlist is None else itemlist
 
     def __reduce__(self):
         # XXX filter itemlist for picklabel items
@@ -457,13 +482,54 @@ class SelectedLegendBoxItem(LegendBoxItem):
 
 class ObjectInfo(object):
     def get_text(self):
-        return u""
+        return ""
+
+class RangeInfo(ObjectInfo):
+    """ObjectInfo handling XRangeSelection shape informations: x, dx
+    
+    label: formatted string
+    xrangeselection: XRangeSelection object
+    function: input arguments are x, dx ; returns objects used to format the 
+    label. Default function is `lambda x, dx: (x, dx)`.
+
+    Example:
+    -------
+    
+    x = linspace(-10, 10, 10)
+    y = sin(sin(sin(x)))
+    xrangeselection = make.range(-2, 2)
+    RangeInfo(u"x = %.1f ± %.1f cm", xrangeselection,
+              lambda x, dx: (x, dx))
+    disp = make.info_label('BL', comp, title="titre")
+    """
+    def __init__(self, label, xrangeselection, function=None):
+        self.label = to_text_string(label)
+        self.range = xrangeselection
+        if function is None:
+            function = lambda x, dx: (x, dx)
+        self.func = function
+
+    def get_text(self):
+        x0, x1 = self.range.get_range()
+        x = .5*(x0+x1)
+        dx = .5*(x1-x0)
+        return self.label % self.func(x, dx)
 
 class RangeComputation(ObjectInfo):
-    def __init__(self, label, curve, range, function):
-        self.label = unicode(label)
+    """ObjectInfo showing curve computations relative to a XRangeSelection 
+    shape.
+    
+    label: formatted string
+    curve: CurveItem object
+    xrangeselection: XRangeSelection object
+    function: input arguments are x, y arrays (extraction of arrays 
+    corresponding to the xrangeselection X-axis range)"""
+    def __init__(self, label, curve, xrangeselection, function=None):
+        self.label = to_text_string(label)
         self.curve = curve
-        self.range = range
+        self.range = xrangeselection
+        if function is None:
+            function = lambda x, dx: (x, dx)
         self.func = function
         
     def set_curve(self, curve):
@@ -490,7 +556,7 @@ class RangeComputation(ObjectInfo):
 
 class RangeComputation2d(ObjectInfo):
     def __init__(self, label, image, rect, function):
-        self.label = unicode(label)
+        self.label = to_text_string(label)
         self.image = image
         self.rect = rect
         self.func = function
@@ -505,15 +571,18 @@ class RangeComputation2d(ObjectInfo):
 class DataInfoLabel(LabelItem):
     __implements__ = (IBasePlotItem,)
 
-    def __init__(self, dataset, infos):
-        super(DataInfoLabel, self).__init__("", dataset)
+    def __init__(self, labelparam=None, infos=None):
+        super(DataInfoLabel, self).__init__(None, labelparam)
         if isinstance(infos, ObjectInfo):
             infos = [infos]
         self.infos = infos
-
+    
     def __reduce__(self):
         return (self.__class__, (self.labelparam, self.infos))
 
+    def types(self):
+        return (IShapeItemType,)
+
     def update_text(self):
         title = self.labelparam.label
         if title:
@@ -522,4 +591,4 @@ class DataInfoLabel(LabelItem):
             text = []
         for info in self.infos:
             text.append(info.get_text())
-        self.set_text( u"<br/>".join(text) )
+        self.set_text( "<br/>".join(text) )
