@@ -100,6 +100,8 @@ Reference
 .. autofunction:: pcolor
 """
 
+from __future__ import print_function
+
 import sys
 from guidata.qt.QtGui import (QMainWindow, QPrinter, QPainter, QFrame,
                               QVBoxLayout, QGridLayout, QToolBar, QPixmap,
@@ -108,11 +110,12 @@ from guidata.qt.QtCore import QRect, Qt, QBuffer, QIODevice
 
 import guidata
 from guidata.configtools import get_icon
+from guidata.py3compat import is_text_string, to_text_string
 
 # Local imports
 from guiqwt.config import _
 from guiqwt.plot import PlotManager
-from guiqwt.image import ImagePlot
+from guiqwt.image import ImagePlot, INTERP_NEAREST, INTERP_LINEAR, INTERP_AA
 from guiqwt.curve import CurvePlot, PlotItemList
 from guiqwt.histogram import ContrastAdjustment
 from guiqwt.cross_section import XCrossSection, YCrossSection
@@ -151,11 +154,11 @@ class Window(QMainWindow):
         self.frame = frame
 
         self.setWindowTitle(wintitle)
-        self.setWindowIcon(get_icon('guiqwt.png'))
+        self.setWindowIcon(get_icon('guiqwt.svg'))
 
     def closeEvent(self, event):
         global _figures, _current_fig, _current_axes
-        figure_title = unicode(self.windowTitle())
+        figure_title = to_text_string(self.windowTitle())
         if _figures.pop(figure_title) == _current_fig:
             _current_fig = None
             _current_axes = None
@@ -173,6 +176,10 @@ class Window(QMainWindow):
     def replot(self):
         for plot in self.plots:
             plot.replot()
+            item = plot.get_default_item()
+            if item is not None:
+                plot.set_active_item(item)
+                item.unselect()
             
     def add_panels(self, images=False):
         self.manager.add_panel(self.itemlist)
@@ -191,6 +198,7 @@ class Window(QMainWindow):
         self.show()
         self.replot()
         self.manager.get_default_tool().activate()
+        self.manager.update_tools_status()
 
 
 class Figure(object):
@@ -201,18 +209,18 @@ class Figure(object):
         self.app = None
 
     def get_axes(self, i, j):
-        if (i,j) in self.axes:
-            return self.axes[(i,j)]
+        if (i, j) in self.axes:
+            return self.axes[(i, j)]
 
         ax = Axes()
-        self.axes[(i,j)] = ax
+        self.axes[(i, j)] = ax
         return ax
 
     def build_window(self):
         self.app = guidata.qapplication()
         self.win = Window(wintitle=self.title)
         images = False
-        for (i, j), ax in self.axes.items():
+        for (i, j), ax in list(self.axes.items()):
             ax.setup_window(i, j, self.win)
             if ax.images:
                 images = True
@@ -225,7 +233,7 @@ class Figure(object):
         self.win.display()
         
     def save(self, fname, format, draft):
-        if isinstance(fname, basestring):
+        if is_text_string(fname):
             if format == "pdf":
                 self.app = guidata.qapplication()
                 if draft:
@@ -263,15 +271,15 @@ class Figure(object):
         W = device.width()
         H = device.height()
         from numpy import array
-        coords = array(self.axes.keys())
-        imin = coords[:,0].min()
-        imax = coords[:,0].max()
-        jmin = coords[:,1].min()
-        jmax = coords[:,1].max()
+        coords = array(list(self.axes.keys()))
+        imin = coords[:, 0].min()
+        imax = coords[:, 0].max()
+        jmin = coords[:, 1].min()
+        jmax = coords[:, 1].max()
         w = W/(jmax-jmin+1)
         h = H/(imax-imin+1)
         paint = QPainter(device)
-        for (i,j), ax in self.axes.items():
+        for (i, j), ax in list(self.axes.items()):
             oy = (i-imin)*h
             ox = (j-jmin)*w
             ax.widget.print_(paint, QRect(ox, oy, w, h))
@@ -280,7 +288,7 @@ class Figure(object):
 def do_mainloop(mainloop):
     global _current_fig
     if not _current_fig:
-        print >>sys.stderr, "Warning: must create a figure before showing it"
+        print("Warning: must create a figure before showing it", file=sys.stderr)
     elif mainloop:
         app = guidata.qapplication()
         app.exec_()
@@ -387,7 +395,7 @@ def _make_figure_title(N=None):
     global _figures
     if N is None:
         N = len(_figures)+1
-    if isinstance(N, basestring):
+    if is_text_string(N):
         return N
     else:
         return "Figure %d" % N
@@ -427,7 +435,7 @@ def show(mainloop=True):
     This should be the last line of your script
     """
     global _figures, _interactive
-    for fig in _figures.values():
+    for fig in list(_figures.values()):
         fig.show()
     if not _interactive:
         do_mainloop(mainloop)
@@ -456,7 +464,7 @@ def subplot(n, m, k):
     lig = (k-1)/m
     col = (k-1)%m
     fig = gcf()
-    axe = fig.get_axes(lig,col)
+    axe = fig.get_axes(lig, col)
     _current_axes = axe
     return axe
 
@@ -594,12 +602,13 @@ def errorbar(*args, **kwargs):
 
 def imread(fname, to_grayscale=False):
     """Read data from *fname*"""
-    from guiqwt.io import imagefile_to_array
-    return imagefile_to_array(fname, to_grayscale=to_grayscale)
+    from guiqwt import io
+    return io.imread(fname, to_grayscale=to_grayscale)
 
-def imshow(data, mask=None):
+def imshow(data, interpolation=None, mask=None):
     """
     Display the image in *data* to current axes
+    interpolation: 'nearest', 'linear' (default), 'antialiasing'
     
     Example:
         
@@ -620,6 +629,12 @@ def imshow(data, mask=None):
         img = make.image(data)
     else:
         img = make.maskedimage(data, mask, show_mask=True)
+    if interpolation is not None:
+        interp_dict = {'nearest': INTERP_NEAREST,
+                       'linear': INTERP_LINEAR,
+                       'antialiasing': INTERP_AA}
+        assert interpolation in interp_dict, "invalid interpolation option"
+        img.set_interpolation(interp_dict[interpolation], size=5)
     axe.add_image(img)
     axe.yreverse = True
     _show_if_interactive()
@@ -674,19 +689,19 @@ def title(text):
 
 def xlabel(bottom="", top=""):
     """Set current x-axis label"""
-    assert isinstance(bottom, basestring) and isinstance(top, basestring)
+    assert is_text_string(bottom) and is_text_string(top)
     axe = gca()
     axe.xlabel = (bottom, top)
 
 def ylabel(left="", right=""):
     """Set current y-axis label"""
-    assert isinstance(left, basestring) and isinstance(right, basestring)
+    assert is_text_string(left) and is_text_string(right)
     axe = gca()
     axe.ylabel = (left, right)
 
 def zlabel(label):
     """Set current z-axis label"""
-    assert isinstance(label, basestring)
+    assert is_text_string(label)
     axe = gca()
     axe.zlabel = label
     
@@ -746,7 +761,7 @@ def savefig(fname, format=None, draft=False):
     
     Currently supports PDF and PNG formats only
     """
-    if not isinstance(fname, basestring) and format is None:
+    if not is_text_string(fname) and format is None:
         # Buffer/fd
         format = 'png'
     if format is None:
