@@ -28,14 +28,17 @@ Reference
 .. autofunction:: exec_images_open_dialog
 """
 
-import sys, os.path as osp
+import sys
+import os.path as osp
+
 from guidata.qt.QtGui import QMessageBox
 from guidata.qt.compat import getsavefilename, getopenfilename, getopenfilenames
 
+from guidata.py3compat import to_text_string
+
 # Local imports
 from guiqwt.config import _
-from guiqwt.io import (IMAGE_SAVE_FILTERS, IMAGE_LOAD_FILTERS,
-                       array_to_imagefile, imagefile_to_array)
+from guiqwt import io
 
 
 #===============================================================================
@@ -55,24 +58,24 @@ def exec_image_save_dialog(data, parent, basedir='', app_name=None):
     saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
     sys.stdout = None
     filename, _filter = getsavefilename(parent, _("Save as"), basedir,
-                                        IMAGE_SAVE_FILTERS)
+                            io.iohandler.get_filters('save', dtype=data.dtype))
     sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
     if filename:
-        filename = unicode(filename)
+        filename = to_text_string(filename)
         try:
-            array_to_imagefile(data, filename)
+            io.imwrite(filename, data)
             return filename
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(parent,
                  _('Error') if app_name is None else app_name,
-                 (_(u"%s could not be written:") % osp.basename(filename))+\
+                 (_("%s could not be written:") % osp.basename(filename))+\
                  "\n"+str(msg))
             return
 
 def exec_image_open_dialog(parent, basedir='', app_name=None,
-                           to_grayscale=True):
+                           to_grayscale=True, dtype=None):
     """
     Executes an image open dialog box (QFileDialog.getOpenFileName)
         * parent: parent widget (None means no parent)
@@ -86,23 +89,23 @@ def exec_image_open_dialog(parent, basedir='', app_name=None,
     saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
     sys.stdout = None
     filename, _filter = getopenfilename(parent, _("Open"), basedir,
-                                        IMAGE_LOAD_FILTERS)
+                                io.iohandler.get_filters('load', dtype=dtype))
     sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
-    filename = unicode(filename)
+    filename = to_text_string(filename)
     try:
-        data = imagefile_to_array(filename, to_grayscale=to_grayscale)
-    except Exception, msg:
+        data = io.imread(filename, to_grayscale=to_grayscale)
+    except Exception as msg:
         import traceback
         traceback.print_exc()
         QMessageBox.critical(parent,
              _('Error') if app_name is None else app_name,
-             (_(u"%s could not be opened:") % osp.basename(filename))+\
+             (_("%s could not be opened:") % osp.basename(filename))+\
              "\n"+str(msg))
         return
     return filename, data
 
 def exec_images_open_dialog(parent, basedir='', app_name=None,
-                            to_grayscale=True):
+                            to_grayscale=True, dtype=None):
     """
     Executes an image*s* open dialog box (QFileDialog.getOpenFileNames)
         * parent: parent widget (None means no parent)
@@ -116,18 +119,18 @@ def exec_images_open_dialog(parent, basedir='', app_name=None,
     saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
     sys.stdout = None
     filenames, _filter = getopenfilenames(parent, _("Open"), basedir,
-                                          IMAGE_LOAD_FILTERS)
+                                io.iohandler.get_filters('load', dtype=dtype))
     sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
-    filenames = [unicode(fname) for fname in list(filenames)]
+    filenames = [to_text_string(fname) for fname in list(filenames)]
     for filename in filenames:
         try:
-            data = imagefile_to_array(filename, to_grayscale=to_grayscale)
-        except Exception, msg:
+            data = io.imread(filename, to_grayscale=to_grayscale)
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(parent,
                  _('Error') if app_name is None else app_name,
-                 (_(u"%s could not be opened:") % osp.basename(filename))+\
+                 (_("%s could not be opened:") % osp.basename(filename))+\
                  "\n"+str(msg))
             return
         yield filename, data
