@@ -74,13 +74,14 @@ Reference
    :inherited-members:
 """
 
-import sys, numpy as np
+import numpy as np
 from math import fabs, sqrt, sin, cos, pi
 
 from guidata.qt.QtGui import QPen, QBrush, QPolygonF, QTransform, QPainter
 from guidata.qt.QtCore import Qt, QRectF, QPointF, QPoint, QLineF
 
 from guidata.utils import assert_interfaces_valid, update_dataset
+from guidata.py3compat import maxsize
 
 # Local imports
 from guiqwt.transitional import QwtPlotItem, QwtSymbol, QwtPlotMarker
@@ -92,6 +93,7 @@ from guiqwt.signals import (SIG_RANGE_CHANGED, SIG_MARKER_CHANGED,
                             SIG_AXES_CHANGED, SIG_ITEM_MOVED)
 from guiqwt.geometry import (vector_norm, vector_projection, vector_rotation,
                              compute_center)
+from guiqwt.baseplot import canvas_to_axes
 
 
 class AbstractShape(QwtPlotItem):
@@ -208,14 +210,14 @@ class AbstractShape(QwtPlotItem):
     def move_local_point_to(self, handle, pos, ctrl=None):
         """Move a handle as returned by hit_test to the new position pos
         ctrl: True if <Ctrl> button is being pressed, False otherwise"""
-        pt = self.canvas_to_axes(pos)
+        pt = canvas_to_axes(self, pos)
         self.move_point_to(handle, pt, ctrl)
         
     def move_local_shape(self, old_pos, new_pos):
         """Translate the shape such that old_pos becomes new_pos
         in canvas coordinates"""
-        old_pt = self.canvas_to_axes(old_pos)
-        new_pt = self.canvas_to_axes(new_pos)
+        old_pt = canvas_to_axes(self, old_pos)
+        new_pt = canvas_to_axes(self, new_pos)
         self.move_shape(old_pt, new_pt)
         if self.plot():
             self.plot().emit(SIG_ITEM_MOVED, self, *(old_pt+new_pt))
@@ -228,16 +230,6 @@ class AbstractShape(QwtPlotItem):
         self.move_shape([0, 0], [delta_x, delta_y])
 
     #------Public API-----------------------------------------------------------
-    def canvas_to_axes(self, pos):
-        plot = self.plot()
-        ax = self.xAxis()
-        ay = self.yAxis()
-        return plot.invTransform(ax, pos.x()), plot.invTransform(ay, pos.y())
-
-    def axes_to_canvas(self, x, y):
-        plot = self.plot()
-        return plot.transform(self.xAxis(), x), plot.transform(self.yAxis(), y)
-
     def move_point_to(self, handle, pos, ctrl=None):
         pass
     
@@ -387,7 +379,7 @@ class Marker(QwtPlotMarker):
         # The following assert has no purpose except reminding that the 
         # markerstyle is one of the MARKERSTYLES dictionary values, in case 
         # this dictionary evolves in the future (this should not fail):
-        assert ms in MARKERSTYLES.values()
+        assert ms in list(MARKERSTYLES.values())
         if ms == "NoLine":
             return sqrt((x-xc)**2 + (y-yc)**2), 0, False, None
         elif ms == "HLine":
@@ -395,7 +387,7 @@ class Marker(QwtPlotMarker):
         elif ms == "VLine":
             return sqrt((x-xc)**2), 0, False, None
         elif ms == "Cross":
-            return sqrt(min((x-xc)**2,(y-yc)**2) ), 0, False, None
+            return sqrt(min((x-xc)**2, (y-yc)**2) ), 0, False, None
         
     def get_item_parameters(self, itemparams):
         """
@@ -422,14 +414,14 @@ class Marker(QwtPlotMarker):
     def move_local_point_to(self, handle, pos, ctrl=None):
         """Move a handle as returned by hit_test to the new position pos
         ctrl: True if <Ctrl> button is being pressed, False otherwise"""
-        x, y = self.canvas_to_axes(pos)
+        x, y = canvas_to_axes(self, pos)
         self.set_pos(x, y)
         
     def move_local_shape(self, old_pos, new_pos):
         """Translate the shape such that old_pos becomes new_pos
         in canvas coordinates"""
-        old_pt = self.canvas_to_axes(old_pos)
-        new_pt = self.canvas_to_axes(new_pos)
+        old_pt = canvas_to_axes(self, old_pos)
+        new_pt = canvas_to_axes(self, new_pos)
         self.move_shape(old_pt, new_pt)
 
     def move_with_selection(self, delta_x, delta_y):
@@ -443,18 +435,6 @@ class Marker(QwtPlotMarker):
     def set_style(self, section, option):
         self.markerparam.read_config(CONF, section, option)
         self.markerparam.update_marker(self)
-
-    def canvas_to_axes(self, pos):
-        plot = self.plot()
-        if plot is None:
-            return
-        ax = self.xAxis()
-        ay = self.yAxis()
-        return plot.invTransform(ax, pos.x()), plot.invTransform(ay, pos.y())
-
-    def axes_to_canvas(self, x, y):
-        plot = self.plot()
-        return plot.transform(self.xAxis(), x), plot.transform(self.yAxis(), y)
         
     def set_pos(self, x=None, y=None):
         if x is None:
@@ -554,11 +534,13 @@ assert_interfaces_valid(Marker)
 
 
 class PolygonShape(AbstractShape):
+    __implements__ = (IBasePlotItem, ISerializableType)
     ADDITIONNAL_POINTS = 0 # Number of points which are not part of the shape
     LINK_ADDITIONNAL_POINTS = False # Link additionnal points with dotted lines
-    def __init__(self, points, closed=True, shapeparam=None):
+    CLOSED = True
+    def __init__(self, points=None, closed=None, shapeparam=None):
         super(PolygonShape, self).__init__()
-        self.closed = closed
+        self.closed = self.CLOSED if closed is None else closed
         self.selected = False
         
         if shapeparam is None:
@@ -574,7 +556,7 @@ class PolygonShape(AbstractShape):
         self.sel_brush = QBrush()
         self.sel_symbol = QwtSymbol.NoSymbol
         self.points = np.zeros( (0, 2), float )
-        if points:
+        if points is not None:
             self.set_points(points)
                 
     def types(self):
@@ -582,16 +564,31 @@ class PolygonShape(AbstractShape):
 
     def __reduce__(self):
         self.shapeparam.update_param(self)
-        state = (self.shapeparam, self.points, self.z())
-        return (PolygonShape, (None, self.closed), state)
+        state = (self.shapeparam, self.points, self.closed, self.z())
+        return (PolygonShape, (), state)
 
     def __setstate__(self, state):
-        param, points, z = state
-        self.points = points
+        self.shapeparam, self.points, self.closed, z = state
         self.setZ(z)
-        self.shapeparam = param
         self.shapeparam.update_shape(self)
     
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        self.shapeparam.update_param(self)
+        writer.write(self.shapeparam, group_name='shapeparam')
+        writer.write(self.points, group_name='points')
+        writer.write(self.closed, group_name='closed')
+        writer.write(self.z(), group_name='z')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        self.closed = reader.read('closed')
+        self.shapeparam = ShapeParam(_("Shape"), icon="rectangle.png")
+        reader.read('shapeparam', instance=self.shapeparam)
+        self.shapeparam.update_shape(self)
+        self.points = reader.read(group_name='points', func=reader.read_array)
+        self.setZ(reader.read('z'))
+    
     #----Public API-------------------------------------------------------------
 
     def set_style(self, section, option):
@@ -610,19 +607,20 @@ class PolygonShape(AbstractShape):
         """Return bounding rectangle coordinates (in plot coordinates)"""
         poly = QPolygonF()
         shape_points = self.points[:-self.ADDITIONNAL_POINTS]
-        for i in xrange(shape_points.shape[0]):
+        for i in range(shape_points.shape[0]):
             poly.append(QPointF(shape_points[i, 0], shape_points[i, 1]))
         return poly.boundingRect().getCoords()
         
     def transform_points(self, xMap, yMap):
         points = QPolygonF()
-        for i in xrange(self.points.shape[0]):
+        for i in range(self.points.shape[0]):
             points.append(QPointF(xMap.transform(self.points[i, 0]),
                                   yMap.transform(self.points[i, 1])))
         return points
     
     def get_reference_point(self):
-        return self.points.mean(axis=0)
+        if self.points.size:
+            return self.points.mean(axis=0)
 
     def get_pen_brush(self, xMap, yMap):
         if self.selected:
@@ -633,20 +631,20 @@ class PolygonShape(AbstractShape):
             pen = self.pen
             brush = self.brush
             sym = self.symbol
-
-        tr = brush.transform()
-        x0, y0 = self.get_reference_point()
-        xx0 = xMap.transform(x0)
-        yy0 = yMap.transform(y0)
-        try:
-            # Optimized version in PyQt >= v4.5
-            t0 = QTransform.fromTranslate(xx0, yy0) 
-        except AttributeError:
-            # Fallback for PyQt <= v4.4
-            t0 = QTransform().translate(xx0, yy0)
-        tr = tr*t0
-        brush = QBrush(brush)
-        brush.setTransform(tr)
+        if self.points.size > 0:
+            x0, y0 = self.get_reference_point()
+            xx0 = xMap.transform(x0)
+            yy0 = yMap.transform(y0)
+            try:
+                # Optimized version in PyQt >= v4.5
+                t0 = QTransform.fromTranslate(xx0, yy0) 
+            except AttributeError:
+                # Fallback for PyQt <= v4.4
+                t0 = QTransform().translate(xx0, yy0)
+            tr = brush.transform()
+            tr = tr*t0
+            brush = QBrush(brush)
+            brush.setTransform(tr)
         return pen, brush, sym
 
     def draw(self, painter, xMap, yMap, canvasRect):
@@ -666,7 +664,7 @@ class PolygonShape(AbstractShape):
         else:
             painter.drawPolyline(shape_points)
         if symbol != QwtSymbol.NoSymbol:
-            for i in xrange(points.size()):
+            for i in range(points.size()):
                 symbol.draw(painter, points[i].toPoint())
         if self.LINK_ADDITIONNAL_POINTS and other_points:
             pen2 = painter.pen()
@@ -676,12 +674,12 @@ class PolygonShape(AbstractShape):
     
     def poly_hit_test(self, plot, ax, ay, pos):
         pos = QPointF(pos)
-        dist = sys.maxint
+        dist = maxsize
         handle = -1
         Cx, Cy = pos.x(), pos.y()
         poly = QPolygonF()
         pts = self.points
-        for i in xrange(pts.shape[0]):
+        for i in range(pts.shape[0]):
             # On calcule la distance dans le repère du canvas
             px = plot.transform(ax, pts[i, 0])
             py = plot.transform(ay, pts[i, 1])
@@ -697,17 +695,17 @@ class PolygonShape(AbstractShape):
     def hit_test(self, pos):
         """return (dist, handle, inside)"""
         if not self.plot():
-            return sys.maxint, 0, False, None
+            return maxsize, 0, False, None
         return self.poly_hit_test(self.plot(), self.xAxis(), self.yAxis(), pos)
     
     def add_local_point(self, pos):
-        pt = self.canvas_to_axes(pos)
+        pt = canvas_to_axes(self, pos)
         return self.add_point(pt)
         
     def add_point(self, pt):
-        N,_ = self.points.shape
+        N, _ = self.points.shape
         self.points = np.resize(self.points, (N+1, 2))
-        self.points[N, :] = pt
+        self.points[N,:] = pt
         return N
 
     def del_point(self, handle):
@@ -718,7 +716,7 @@ class PolygonShape(AbstractShape):
             return self.points.shape[0]-1
     
     def move_point_to(self, handle, pos, ctrl=None):
-        self.points[handle, :] = pos
+        self.points[handle,:] = pos
         
     def move_shape(self, old_pos, new_pos):
         dx = new_pos[0]-old_pos[0]
@@ -738,9 +736,9 @@ assert_interfaces_valid(PolygonShape)
 
 
 class PointShape(PolygonShape):
-    def __init__(self, x, y, shapeparam=None):
-        super(PointShape, self).__init__([], closed=False,
-                                         shapeparam=shapeparam)
+    CLOSED = False
+    def __init__(self, x=0, y=0, shapeparam=None):
+        super(PointShape, self).__init__(shapeparam=shapeparam)
         self.set_pos(x, y)
         
     def set_pos(self, x, y):
@@ -757,16 +755,21 @@ class PointShape(PolygonShape):
 
     def __reduce__(self):
         state = (self.shapeparam, self.points, self.z())
-        return (self.__class__, (0, 0), state)
+        return (self.__class__, (), state)
+
+    def __setstate__(self, state):
+        self.shapeparam, self.points, z = state
+        self.setZ(z)
+        self.shapeparam.update_shape(self)
 
 assert_interfaces_valid(PointShape)
 
 
 class SegmentShape(PolygonShape):
+    CLOSED = False
     ADDITIONNAL_POINTS = 1 # Number of points which are not part of the shape
-    def __init__(self, x1, y1, x2, y2, shapeparam=None):
-        super(SegmentShape, self).__init__([], closed=False,
-                                           shapeparam=shapeparam)
+    def __init__(self, x1=0, y1=0, x2=0, y2=0, shapeparam=None):
+        super(SegmentShape, self).__init__(shapeparam=shapeparam)
         self.set_rect(x1, y1, x2, y2)
         
     def set_rect(self, x1, y1, x2, y2):
@@ -793,7 +796,7 @@ class SegmentShape(PolygonShape):
 
     def __reduce__(self):
         state = (self.shapeparam, self.points, self.z())
-        return (self.__class__, (0, 0, 0, 0), state)
+        return (self.__class__, (), state)
 
     def __setstate__(self, state):
         param, points, z = state
@@ -815,9 +818,9 @@ assert_interfaces_valid(SegmentShape)
 
 
 class RectangleShape(PolygonShape):
-    def __init__(self, x1, y1, x2, y2, shapeparam=None):
-        super(RectangleShape, self).__init__([], closed=True,
-                                             shapeparam=shapeparam)
+    CLOSED = True
+    def __init__(self, x1=0, y1=0, x2=0, y2=0, shapeparam=None):
+        super(RectangleShape, self).__init__(shapeparam=shapeparam)
         self.set_rect(x1, y1, x2, y2)
         
     def set_rect(self, x1, y1, x2, y2):
@@ -851,7 +854,12 @@ class RectangleShape(PolygonShape):
 
     def __reduce__(self):
         state = (self.shapeparam, self.points, self.z())
-        return (self.__class__, (0,0,0,0), state)
+        return (self.__class__, (), state)
+
+    def __setstate__(self, state):
+        self.shapeparam, self.points, z = state
+        self.setZ(z)
+        self.shapeparam.update_shape(self)
 
 assert_interfaces_valid(RectangleShape)
 
@@ -862,16 +870,16 @@ def _no_null_vector(x0, y0, x1, y1, x2, y2, x3, y3):
            vector_norm(x1, y1, x3, y3) and vector_norm(x2, y2, x3, y3)
     
 class ObliqueRectangleShape(PolygonShape):
+    CLOSED = True
     ADDITIONNAL_POINTS = 2 # Number of points which are not part of the shape
     LINK_ADDITIONNAL_POINTS = True # Link additionnal points with dotted lines
-    def __init__(self, x0, y0, x1, y1, x2, y2, x3, y3,
+    def __init__(self, x0=0, y0=0, x1=0, y1=0, x2=0, y2=0, x3=0, y3=0,
                  shapeparam=None):
-        super(ObliqueRectangleShape, self).__init__([], closed=True,
-                                                    shapeparam=shapeparam)
+        super(ObliqueRectangleShape, self).__init__(shapeparam=shapeparam)
         self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
         
     def set_rect(self, x0, y0, x1, y1, x2, y2, x3, y3):
-        u"""
+        """
         Set the rectangle corners coordinates:
             (x0, y0): top-left corner
             (x1, y1): top-right corner
@@ -984,17 +992,21 @@ class ObliqueRectangleShape(PolygonShape):
 
     def __reduce__(self):
         state = (self.shapeparam, self.points, self.z())
-        return (self.__class__, (0, 0, 0, 0, 0, 0, 0, 0), state)
+        return (self.__class__, (), state)
+
+    def __setstate__(self, state):
+        self.shapeparam, self.points, z = state
+        self.setZ(z)
+        self.shapeparam.update_shape(self)
 
 assert_interfaces_valid(ObliqueRectangleShape)
 
 
 #FIXME: EllipseShape's ellipse drawing is invalid when aspect_ratio != 1
 class EllipseShape(PolygonShape):
-    def __init__(self, x1, y1, x2, y2, ratio=None, shapeparam=None):
-        self.ratio = ratio
-        super(EllipseShape, self).__init__([], closed=True,
-                                           shapeparam=shapeparam)
+    CLOSED = True
+    def __init__(self, x1=0, y1=0, x2=0, y2=0, shapeparam=None):
+        super(EllipseShape, self).__init__(shapeparam=shapeparam)
         self.is_ellipse = False
         self.set_xdiameter(x1, y1, x2, y2)
         
@@ -1008,8 +1020,8 @@ class EllipseShape(PolygonShape):
         yline.translate(xline.pointAt(.5)-xline.p1())
         if self.is_ellipse:
             yline.setLength(self.get_yline().length())
-        elif self.ratio is not None:
-            yline.setLength(xline.length()*self.ratio)
+        else:
+            yline.setLength(xline.length())
         yline.translate(yline.pointAt(.5)-yline.p2())
         self.set_points([(x0, y0), (x1, y1),
                          (yline.x1(), yline.y1()), (yline.x2(), yline.y2())])
@@ -1062,7 +1074,7 @@ class EllipseShape(PolygonShape):
     def hit_test(self, pos):
         """return (dist, handle, inside)"""
         if not self.plot():
-            return sys.maxint, 0, False, None
+            return maxsize, 0, False, None
         dist, handle, inside, other = self.poly_hit_test(self.plot(),
                                              self.xAxis(), self.yAxis(), pos)
         if not inside:
@@ -1087,7 +1099,7 @@ class EllipseShape(PolygonShape):
         painter.drawEllipse(rect.toRect())
         painter.restore()
         if symbol != QwtSymbol.NoSymbol:
-            for i in xrange(points.size()):
+            for i in range(points.size()):
                 symbol.draw(painter, points[i].toPoint())
 
     def get_xline(self):
@@ -1132,15 +1144,22 @@ class EllipseShape(PolygonShape):
 
     def __reduce__(self):
         state = (self.shapeparam, self.points, self.z())
-        return (self.__class__, (0,0,0,0), state)
+        return (self.__class__, (), state)
+
+    def __setstate__(self, state):
+        self.shapeparam, self.points, z = state
+        self.setZ(z)
+        self.shapeparam.update_shape(self)
 
 assert_interfaces_valid(EllipseShape)
 
 
 class Axes(PolygonShape):
     """Axes( (0,1), (1,1), (0,0) )"""
-    def __init__(self, p0, p1, p2, axesparam=None, shapeparam=None):
-        super(Axes, self).__init__([], closed=True, shapeparam=shapeparam)
+    CLOSED = True
+    def __init__(self, p0=(0, 0), p1=(0, 0), p2=(0, 0),
+                 axesparam=None, shapeparam=None):
+        super(Axes, self).__init__(shapeparam=shapeparam)
         self.set_rect(p0, p1, p2)
         self.arrow_angle = 15 # degrees
         self.arrow_size = 0.05 # % of axe length
@@ -1155,8 +1174,9 @@ class Axes(PolygonShape):
         self.axesparam.update_param(self)
 
     def __reduce__(self):
+        self.axesparam.update_param(self)
         state = (self.shapeparam, self.axesparam, self.points, self.z())
-        return (self.__class__, ((0, 0), (0, 0), (0, 0)), state)
+        return (self.__class__, (), state)
 
     def __setstate__(self, state):
         shapeparam, axesparam, points, z = state
@@ -1167,6 +1187,19 @@ class Axes(PolygonShape):
         self.axesparam = axesparam
         self.axesparam.update_axes(self)
         
+    def serialize(self, writer):
+        """Serialize object to HDF5 writer"""
+        super(Axes, self).serialize(writer)
+        self.axesparam.update_param(self)
+        writer.write(self.axesparam, group_name='axesparam')
+    
+    def deserialize(self, reader):
+        """Deserialize object from HDF5 reader"""
+        super(Axes, self).deserialize(reader)
+        self.axesparam = AxesShapeParam(_("Axes"), icon="gtaxes.png")
+        reader.read('axesparam', instance=self.axesparam)
+        self.axesparam.update_axes(self)
+    
     def get_transform_matrix(self, dx=1., dy=1.):
         p0, p1, _p3, p2 = [np.array([p[0], p[1], 1.0]) for p in self.points]
         matrix = np.array([(p1-p0)/dx, (p2-p0)/dy, p0])
@@ -1334,8 +1367,8 @@ class XRangeSelection(AbstractShape):
                          rct2.center().x(), rct2.bottom())
         painter.setPen(pen)
         x0, x1, y = self.get_handles_pos()        
-        sym.draw(painter, QPoint(x0,y))
-        sym.draw(painter, QPoint(x1,y))
+        sym.draw(painter, QPoint(x0, y))
+        sym.draw(painter, QPoint(x1, y))
         
     def hit_test(self, pos):
         x, _y = pos.x(), pos.y()
@@ -1352,8 +1385,8 @@ class XRangeSelection(AbstractShape):
     def move_local_point_to(self, handle, pos, ctrl=None):
         """Move a handle as returned by hit_test to the new position pos
         ctrl: True if <Ctrl> button is being pressed, False otherwise"""
-        val = self.plot().invTransform(self.xAxis(), pos.x())
-        self.move_point_to(handle, (val, 0))
+        x, _y = canvas_to_axes(self, pos)
+        self.move_point_to(handle, (x, 0))
         
     def move_point_to(self, hnd, pos, ctrl=None):
         val, _ = pos
