@@ -116,6 +116,7 @@ from guidata.dataset.dataitems import (ChoiceItem, BoolItem, FloatItem, IntItem,
 from guidata.dataset.qtwidgets import DataSetEditLayout
 from guidata.dataset.qtitemwidgets import DataSetWidget
 from guidata.utils import update_dataset
+from guidata.py3compat import to_text_string
 
 # Local imports
 from guiqwt.transitional import QwtPlot, QwtPlotCurve, QwtSymbol, QwtPlotMarker
@@ -163,8 +164,8 @@ class ItemParameters(object):
         self.__add(key, item, param)
         
     def get(self, key):
-        from copy import copy
-        return copy(self.paramdict.get(key))
+        from copy import deepcopy
+        return deepcopy(self.paramdict.get(key))
     
     def update(self, plot):
         #FIXME: without the following workaround, ImagePlot object aspect ratio
@@ -187,49 +188,49 @@ class ItemParameters(object):
         for key in self.ENDING_PARAMETERS:
             if key in paramdict:
                 ending_parameters.append(paramdict.pop(key))
-        parameters = paramdict.values()+ending_parameters
+        parameters = list(paramdict.values())+ending_parameters
         dset = DataSetGroup(parameters, title=title.rstrip('.'), icon=icon)
         if dset.edit(parent=plot, apply=lambda dset: self.update(plot)):
             self.update(plot)
 
 
 LINESTYLES = {
-              "-"  : "SolidLine",
-              "--" : "DashLine",
-              ":"  : "DotLine",
-              "-." : "DashDotLine",
+              "-": "SolidLine",
+              "--": "DashLine",
+              ":": "DotLine",
+              "-.": "DashDotLine",
               }
 COLORS = {
-          "r" : "red",
-          "g" : "green",
-          "b" : "blue",
-          "c" : "cyan",
-          "m" : "magenta",
-          "y" : "yellow",
-          "k" : "black",
-          "w" : "white",
-          "G" : "gray",
+          "r": "red",
+          "g": "green",
+          "b": "blue",
+          "c": "cyan",
+          "m": "magenta",
+          "y": "yellow",
+          "k": "black",
+          "w": "white",
+          "G": "gray",
           }
 MARKERS = {
-          "+" : QwtSymbol.Cross,
-          "o" : QwtSymbol.Ellipse,
-          "*" : QwtSymbol.Star1,
-          "." : QwtSymbol(QwtSymbol.Ellipse, QBrush(Qt.black),
+          "+": QwtSymbol.Cross,
+          "o": QwtSymbol.Ellipse,
+          "*": QwtSymbol.Star1,
+          ".": QwtSymbol(QwtSymbol.Ellipse, QBrush(Qt.black),
                           QPen(Qt.black), QSize(3, 3)),
-          "x" : QwtSymbol.XCross,
-          "s" : QwtSymbol.Rect,
-          "d" : QwtSymbol.Diamond,
-          "^" : QwtSymbol.UTriangle,
-          "v" : QwtSymbol.DTriangle,
-          ">" : QwtSymbol.RTriangle,
-          "<" : QwtSymbol.LTriangle,
-          "h" : QwtSymbol.Star2,
+          "x": QwtSymbol.XCross,
+          "s": QwtSymbol.Rect,
+          "d": QwtSymbol.Diamond,
+          "^": QwtSymbol.UTriangle,
+          "v": QwtSymbol.DTriangle,
+          ">": QwtSymbol.RTriangle,
+          "<": QwtSymbol.LTriangle,
+          "h": QwtSymbol.Star2,
           }
 MARKERSTYLES = {
                 None: "NoLine",
-                "-" : "HLine",
-                "|" : "VLine",
-                "+" : "Cross",
+                "-": "HLine",
+                "|": "VLine",
+                "+": "Cross",
                 }
 
 
@@ -245,19 +246,19 @@ def update_style_attr(style, param):
     update the color, linestyle, marker attributes of the param
     object
     """
-    for marker in MARKERS.keys():
+    for marker in list(MARKERS.keys()):
         if marker in style:
             param.symbol.update_param(MARKERS[marker])
             break
     else:
         param.symbol.update_param(QwtSymbol.NoSymbol)
-    for linestyle in LINESTYLES.keys():
+    for linestyle in list(LINESTYLES.keys()):
         if linestyle in style:
             param.line.style = LINESTYLES[linestyle]
             break
     else:
         param.line.style = "NoPen"
-    for color in COLORS.keys():
+    for color in list(COLORS.keys()):
         if color in style:
             param.line.color = COLORS[color]
             param.symbol.facecolor = COLORS[color]
@@ -487,7 +488,7 @@ class BrushStyleParam(DataSet):
     def build_brush(self):
         color = QColor(self.color)
         color.setAlphaF(self.alpha)
-        brush = QBrush(color, getattr(Qt,self.style))
+        brush = QBrush(color, getattr(Qt, self.style))
         tr = QTransform()
         tr = tr.scale(self.sx, self.sy)
         tr = tr.rotate(self.angle)
@@ -586,8 +587,8 @@ class GridParam(DataSet):
 # Axes style parameters
 # ===================================================
 class AxeStyleParam(DataSet):
-    title = StringItem(_("Title"), default=u"")
-    unit = StringItem(_("Unit"), default=u"")
+    title = StringItem(_("Title"), default="")
+    unit = StringItem(_("Unit"), default="")
     color = ColorItem(_("Color"), default="black").set_pos(col=1)
     title_font = FontItem(_("Title font"))
     ticks_font = FontItem(_("Values font"))
@@ -719,22 +720,22 @@ class LabelParam(DataSet):
     _begin_anchor = BeginGroup(_("Position relative to anchor")) \
                     .set_prop("display", hide=GetAttrProp("_multiselection"))
     anchor = ChoiceItem(_("Corner"),
-                        [("TL" , _("Top left") ),
-                         ("TR" , _("Top right") ),
-                         ("BL" , _("Bottom left") ),
-                         ("BR" , _("Bottom right") ),
-                         ("L"  , _("Left") ),
-                         ("R"  , _("Right") ),
-                         ("T"  , _("Top") ),
-                         ("B"  , _("Bottom") ),
-                         ("C"  , _("Center") ),], default="TL",
+                        [("TL", _("Top left") ),
+                         ("TR", _("Top right") ),
+                         ("BL", _("Bottom left") ),
+                         ("BR", _("Bottom right") ),
+                         ("L", _("Left") ),
+                         ("R", _("Right") ),
+                         ("T", _("Top") ),
+                         ("B", _("Bottom") ),
+                         ("C", _("Center") ),], default="TL",
                          help=_("Label position relative to anchor point")) \
                          .set_prop("display",
                                    hide=GetAttrProp("_multiselection"))
-    xc = IntItem(_(u"ΔX"), default=5,
+    xc = IntItem(_("ΔX"), default=5,
                  help=_("Horizontal offset (pixels) relative to anchor point"))\
                  .set_prop("display", hide=GetAttrProp("_multiselection"))
-    yc = IntItem(_(u"ΔY"), default=5,
+    yc = IntItem(_("ΔY"), default=5,
                  help=_("Vertical offset (pixels) relative to anchor point")
                  ).set_pos(col=1).set_prop("display",
                                            hide=GetAttrProp("_multiselection"))
@@ -756,7 +757,7 @@ class LabelParam(DataSet):
                    ).set_pos(col=1) \
                     .set_prop("display", active=NotProp(_abspos_prop)) \
                     .set_prop("display", hide=GetAttrProp("_multiselection"))
-    move_anchor = ChoiceItem(_(u"Interact"),
+    move_anchor = ChoiceItem(_("Interact"),
                          ((True, _("moving object changes anchor position")),
                           (False, _("moving object changes label position"))),
                          default=True
@@ -768,11 +769,11 @@ class LabelParam(DataSet):
                          ("TR", _("Top right") ),
                          ("BL", _("Bottom left") ),
                          ("BR", _("Bottom right") ),
-                         ("L" , _("Left") ),
-                         ("R" , _("Right") ),
-                         ("T" , _("Top") ),
-                         ("B" , _("Bottom") ),
-                         ("C" , _("Center") ),], default="TL",
+                         ("L", _("Left") ),
+                         ("R", _("Right") ),
+                         ("T", _("Top") ),
+                         ("B", _("Bottom") ),
+                         ("C", _("Center") ),], default="TL",
                          help=_("Absolute position on canvas")
                          ).set_prop("display", active=_abspos_prop) \
                           .set_prop("display",
@@ -974,7 +975,7 @@ class BaseImageParam(DataSet):
     _end_formats = EndGroup(_("Statistics string formatting"))
                                
     def update_param(self, image):
-        self.label = unicode(image.title().text())
+        self.label = to_text_string(image.title().text())
         self.colormap = image.get_color_map_name()
         interpolation = image.get_interpolation()
         mode = interpolation[0]
@@ -1022,14 +1023,14 @@ class QuadGridParam(DataSet):
                                       "Flat mode use fixed u,v "
                                       "interpolation parameters"))
     uflat = FloatItem(_("Fixed U interpolation parameter"),
-                      default=0.5,min=0.,max=1., help=_("For flat mode only"))
+                      default=0.5, min=0., max=1., help=_("For flat mode only"))
     vflat = FloatItem(_("Fixed V interpolation parameter"),
-                      default=0.5,min=0.,max=1., help=_("For flat mode only"))
+                      default=0.5, min=0., max=1., help=_("For flat mode only"))
     grid = BoolItem(_("Show grid"), default=False)
     gridcolor = ColorItem(_("Grid lines color"), default="black")
                                
     def update_param(self, image):
-        self.label = unicode(image.title().text())
+        self.label = to_text_string(image.title().text())
         self.colormap = image.get_color_map_name()
         interp, uflat, vflat = image.interpolate
         self.interpolation = interp
@@ -1040,7 +1041,7 @@ class QuadGridParam(DataSet):
     def update_image(self, image):
         image.setTitle(self.label)
         image.set_color_map(self.colormap)
-        image.interpolate = (self.interpolation,self.uflat,self.vflat)
+        image.interpolate = (self.interpolation, self.uflat, self.vflat)
         image.grid = self.grid
         # TODO : gridcolor
 
@@ -1165,7 +1166,7 @@ class ImageFilterParam(BaseImageParam):
     def update_param(self, obj):
         self.xmin, self.ymin, self.xmax, self.ymax = obj.border_rect.get_rect()
         self.use_source_cmap = obj.use_source_cmap
-        super(ImageFilterParam,self).update_param(obj)
+        super(ImageFilterParam, self).update_param(obj)
     
     def update_imagefilter(self, imagefilter):
         m, M = imagefilter.get_lut_range()
@@ -1374,7 +1375,7 @@ class MarkerParam(DataSet):
         self.markerstyle = MARKERSTYLES.get(style, style)
 
 class ShapeParam(DataSet):
-    label = StringItem(_("Title"), default=u"")
+    label = StringItem(_("Title"), default="")
     _styles = BeginTabGroup("Styles")
     #------------------------------------------------------------------ Line tab
     ___line = BeginGroup(_("Line")).set_prop("display", icon="dashdot.png")
@@ -1402,7 +1403,7 @@ class ShapeParam(DataSet):
                                "the item list panel")).set_pos(col=1)
     
     def update_param(self, obj):
-        self.label = unicode(obj.title().text())
+        self.label = to_text_string(obj.title().text())
         self.line.update_param(obj.pen)
         self.symbol.update_param(obj.symbol)
         self.fill.update_param(obj.brush)
@@ -1461,8 +1462,8 @@ class AnnotationParam(DataSet):
     show_label = BoolItem(_("Show annotation"), default=True)
     show_computations = BoolItem(_("Show informations on area "
                                    "covered by this shape"), default=True)
-    title = StringItem(_("Title"), default=u"")
-    subtitle = StringItem(_("Subtitle"), default=u"")
+    title = StringItem(_("Title"), default="")
+    subtitle = StringItem(_("Subtitle"), default="")
     format = StringItem(_("String formatting"), default="%.1f")
     uncertainty = FloatItem(_("Uncertainty"), default=0., min=0., max=1.,
                             help=_("Measurement relative uncertainty")
@@ -1479,7 +1480,7 @@ class AnnotationParam(DataSet):
     def update_param(self, obj):
         self.show_label = obj.is_label_visible()
         self.show_computations = obj.area_computations_visible
-        self.title = unicode(obj.title().text())
+        self.title = to_text_string(obj.title().text())
         self.readonly = obj.is_readonly()
         self.private = obj.is_private()
         
