@@ -27,16 +27,16 @@ def test():
     _app = guidata.qapplication()
     # --
     from numpy import linspace, sin
-    x = linspace(-10, 10, 1000)
-    y = sin(sin(sin(x)))
+    x = linspace(-10, 10, 1000)+1
+    y = sin(sin(sin(x)))+3
 
     curve = make.curve(x, y, "ab", "b")
-    hcursor = make.hcursor(.2)
-    vcursor = make.vcursor(2)
-    hcursor_info = make.info_cursor(hcursor, "TL")
-    vcursor_info = make.info_cursor(vcursor, "BR")
+    hcursor = make.hcursor(3.2, label='y = %.2f')
+    vcursor = make.vcursor(7,  label='x = %.2f')
+    vcursor2 = make.vcursor(-1,  label='NOT MOVABLE = %.2f', movable=False)
+    xcursor = make.xcursor(-4, 2.5,  label='x = %.2f<br>y = %.2f')
     legend = make.legend("TR")
-    plot(curve, hcursor, vcursor, hcursor_info, vcursor_info, legend)
+    plot(curve, hcursor, vcursor, vcursor2, xcursor, legend)
 
 if __name__ == "__main__":
     test()
