@@ -44,7 +44,7 @@ class FilterTestWidget(QWidget):
         curvewidget.plot.set_antialiasing(True)
         #---
         
-        button = QPushButton(u"Test filter: %s" % title)
+        button = QPushButton("Test filter: %s" % title)
         self.connect(button, SIGNAL('clicked()'), self.process_data)
         vlayout = QVBoxLayout()
         vlayout.addWidget(curvewidget)
