@@ -47,7 +47,7 @@ class FilterTestWidget(QWidget):
         self.plot.set_antialiasing(True)
         #---
         
-        button = QPushButton(u"Test filter: %s" % title)
+        button = QPushButton("Test filter: %s" % title)
         self.connect(button, SIGNAL('clicked()'), self.process_data)
         vlayout = QVBoxLayout()
         vlayout.addWidget(self.plot)
@@ -71,7 +71,7 @@ class TestWindow(QMainWindow):
     def __init__(self):
         QMainWindow.__init__(self)
         self.setWindowTitle("Signal filtering 2 (guiqwt)")
-        self.setWindowIcon(get_icon('guiqwt.png'))
+        self.setWindowIcon(get_icon('guiqwt.svg'))
         
         hlayout = QHBoxLayout()
         central_widget = QWidget(self)
