@@ -10,6 +10,8 @@ SIFT, the Signal and Image Filtering Tool
 Simple signal and image processing application based on guiqwt and guidata
 """
 
+from __future__ import unicode_literals
+
 SHOW = True # Show test in GUI-based test launcher
 
 from guidata.qt.QtGui import (QMainWindow, QMessageBox, QSplitter, QListWidget,
@@ -18,7 +20,10 @@ from guidata.qt.QtGui import (QMainWindow, QMessageBox, QSplitter, QListWidget,
 from guidata.qt.QtCore import Qt, QT_VERSION_STR, PYQT_VERSION_STR, SIGNAL
 from guidata.qt.compat import getopenfilenames, getsavefilename
 
-import sys, platform, os.path as osp, os
+import sys
+import platform
+import os.path as osp
+import os
 import numpy as np
 
 from guidata.dataset.datatypes import DataSet, ValueProp
@@ -30,6 +35,7 @@ from guidata.configtools import get_icon
 from guidata.qthelpers import create_action, add_actions, get_std_icon
 from guidata.qtwidgets import DockableWidget, DockableWidgetMixin
 from guidata.utils import update_dataset
+from guidata.py3compat import to_text_string
 
 from guiqwt.config import _
 from guiqwt.plot import CurveWidget, ImageWidget
@@ -329,11 +335,11 @@ class ObjectFT(QSplitter):
                     sumobj.copy_data_from(obj)
                 else:
                     sumobj.data += obj.data
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(self.parent(), APP_NAME,
-                                 _(u"Error:")+"\n%s" % str(msg))
+                                 _("Error:")+"\n%s" % str(msg))
             return
         self.add_object(sumobj)
     
@@ -350,11 +356,11 @@ class ObjectFT(QSplitter):
                     sumobj.copy_data_from(obj, dtype=np.float64)
                 else:
                     sumobj.data += obj.data
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(self.parent(), APP_NAME,
-                                 _(u"Error:")+"\n%s" % str(msg))
+                                 _("Error:")+"\n%s" % str(msg))
             return
         sumobj.data /= float(len(rows))
         sumobj.change_data_type(dtype=original_dtype)
@@ -371,11 +377,11 @@ class ObjectFT(QSplitter):
                     sumobj.copy_data_from(obj)
                 else:
                     sumobj.data *= obj.data
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(self.parent(), APP_NAME,
-                                 _(u"Error:")+"\n%s" % str(msg))
+                                 _("Error:")+"\n%s" % str(msg))
             return
         self.add_object(sumobj)
     
@@ -388,11 +394,11 @@ class ObjectFT(QSplitter):
             obj0, obj1 = self.objects[rows[0]], self.objects[rows[1]]
             diffobj.copy_data_from(obj0)
             diffobj.data = obj0.data-obj1.data
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(self.parent(), APP_NAME,
-                                 _(u"Error:")+"\n%s" % str(msg))
+                                 _("Error:")+"\n%s" % str(msg))
             return
         self.add_object(diffobj)
     
@@ -405,11 +411,11 @@ class ObjectFT(QSplitter):
             obj0, obj1 = self.objects[rows[0]], self.objects[rows[1]]
             diffobj.copy_data_from(obj0)
             diffobj.data = obj0.data/obj1.data
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(self.parent(), APP_NAME,
-                                 _(u"Error:")+"\n%s" % str(msg))
+                                 _("Error:")+"\n%s" % str(msg))
             return
         self.add_object(diffobj)
                                      
@@ -442,11 +448,11 @@ class ObjectFT(QSplitter):
             self.repaint()
             try:
                 self.apply_11_func(obj, orig, func, param)
-            except Exception, msg:
+            except Exception as msg:
                 import traceback
                 traceback.print_exc()
                 QMessageBox.critical(self.parent(), APP_NAME,
-                                     _(u"Error:")+"\n%s" % str(msg))
+                                     _("Error:")+"\n%s" % str(msg))
                 return
             finally:
                 self.emit(SIGNAL("status_message(QString)"), "")
@@ -518,7 +524,7 @@ class SignalFT(ObjectFT):
             x, y, dx, dy = data
             item = make.merror(x, y.real, dx, dy, label=signal.title)
         else:
-            raise RuntimeError, "data not supported"
+            raise RuntimeError("data not supported")
         self.items[row] = item
         return item
         
@@ -543,7 +549,7 @@ class SignalFT(ObjectFT):
         self.compute_11("ROI", lambda x, y, p: (x.copy()[p.row1:p.row2],
                                                 y.copy()[p.row1:p.row2]),
                         param, suffix=lambda p:
-                                      u"rows=%d:%d" % (p.row1, p.row2))
+                                      "rows=%d:%d" % (p.row1, p.row2))
     
     def swap_axes(self):
         self.compute_11("SwapAxes", lambda x, y: (y, x))
@@ -578,7 +584,7 @@ class SignalFT(ObjectFT):
         def func(x, y, p):
             return x, normalize(y, p.method)
         self.compute_11("Normalize", func, param,
-                        suffix=lambda p: u"ref=%s" % p.method)
+                        suffix=lambda p: "ref=%s" % p.method)
     
     def calibrate(self):
         axes = (('x', _("X-axis")), ('y', _("Y-axis")))
@@ -593,8 +599,8 @@ class SignalFT(ObjectFT):
             else:
                 return x, p.a*y+p.b
         self.compute_11("LinearCal", func, param,
-                        suffix=lambda p: u"%s=%s*%s+%s" % (p.axis, p.a,
-                                                           p.axis, p.b))
+                        suffix=lambda p: "%s=%s*%s+%s" % (p.axis, p.a,
+                                                          p.axis, p.b))
     
     def compute_wiener(self):
         import scipy.signal as sps
@@ -604,13 +610,13 @@ class SignalFT(ObjectFT):
     
     def compute_gaussian(self):
         class GaussianParam(DataSet):
-            sigma = FloatItem(u"σ", default=1.)
+            sigma = FloatItem("σ", default=1.)
         param = GaussianParam(_("Gaussian filter"))
         import scipy.ndimage as spi
         def func(x, y, p):
             return x, spi.gaussian_filter1d(y, p.sigma)
         self.compute_11("GaussianFilter", func, param,
-                        suffix=lambda p: u"σ=%.3f pixels" % p.sigma)
+                        suffix=lambda p: "σ=%.3f pixels" % p.sigma)
                          
     def compute_fft(self):
         self.compute_11("FFT", xy_fft)
@@ -640,7 +646,7 @@ class SignalFT(ObjectFT):
             class GaussParam(DataSet):
                 a = FloatItem("Norm", default=1.)
                 x0 = FloatItem("X0", default=0.0)
-                sigma = FloatItem(u"σ", default=5.)
+                sigma = FloatItem("σ", default=5.)
             param = GaussParam(_("New gaussian function"))
             if not param.edit(parent=self.parent()):
                 return
@@ -653,13 +659,13 @@ class SignalFT(ObjectFT):
         saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
         sys.stdout = None
         filters = '%s (*.txt *.csv)\n%s (*.npy)'\
-                  % (_(u"Text files"), _(u"NumPy arrays"))
+                  % (_("Text files"), _("NumPy arrays"))
         filenames, _filter = getopenfilenames(self.parent(), _("Open"), '',
                                               filters)
         sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
         filenames = list(filenames)
         for filename in filenames:
-            filename = unicode(filename)
+            filename = to_text_string(filename)
             os.chdir(osp.dirname(filename))
             signal = SignalParam()
             signal.title = filename
@@ -676,11 +682,11 @@ class SignalFT(ObjectFT):
                     else:
                         raise
                 assert len(xydata.shape) in (1, 2), "Data not supported"
-            except Exception, msg:
+            except Exception as msg:
                 import traceback
                 traceback.print_exc()
                 QMessageBox.critical(self.parent(), APP_NAME,
-                     (_(u"%s could not be opened:") % osp.basename(filename))+\
+                     (_("%s could not be opened:") % osp.basename(filename))+\
                      "\n"+str(msg))
                 return
             if len(xydata.shape) == 1:
@@ -704,19 +710,19 @@ class SignalFT(ObjectFT):
         rows = self._get_selected_rows()
         for row in rows:
             filename, _filter = getsavefilename(self, _("Save as"), '',
-                                                _(u"CSV files")+" (*.csv)")
+                                                _("CSV files")+" (*.csv)")
             if not filename:
                 return
-            filename = unicode(filename)
+            filename = to_text_string(filename)
             os.chdir(osp.dirname(filename))
             obj = self.objects[row]
             try:
                 np.savetxt(filename, obj.xydata, delimiter=',')
-            except Exception, msg:
+            except Exception as msg:
                 import traceback
                 traceback.print_exc()
                 QMessageBox.critical(self.parent(), APP_NAME,
-                     (_(u"%s could not be written:") % osp.basename(filename))+\
+                     (_("%s could not be written:") % osp.basename(filename))+\
                      "\n"+str(msg))
                 return
 
@@ -821,20 +827,20 @@ class ImageFT(ObjectFT):
         boundaries = ('constant', 'nearest', 'reflect', 'wrap')
         prop = ValueProp(False)
         class RotateParam(DataSet):
-            angle = FloatItem(u"%s (°)" % _(u"Angle"))
-            mode = ChoiceItem(_(u"Mode"), zip(boundaries, boundaries),
+            angle = FloatItem("%s (°)" % _("Angle"))
+            mode = ChoiceItem(_("Mode"), list(zip(boundaries, boundaries)),
                               default=boundaries[0])
             cval = FloatItem(_("cval"), default=0.,
-                             help=_(u"Value used for points outside the "
-                                    u"boundaries of the input if mode is "
-                                    u"'constant'"))
-            reshape = BoolItem(_(u"Reshape the output array"), default=True,
-                               help=_(u"Reshape the output array "
-                                      u"so that the input array is "
-                                      u"contained completely in the output"))
-            prefilter = BoolItem(_(u"Prefilter the input image"),
+                             help=_("Value used for points outside the "
+                                    "boundaries of the input if mode is "
+                                    "'constant'"))
+            reshape = BoolItem(_("Reshape the output array"), default=True,
+                               help=_("Reshape the output array "
+                                      "so that the input array is "
+                                      "contained completely in the output"))
+            prefilter = BoolItem(_("Prefilter the input image"),
                                  default=True).set_prop("display", store=prop)
-            order = IntItem(_(u"Order"), default=3, min=0, max=5,
+            order = IntItem(_("Order"), default=3, min=0, max=5,
                             help=_("Spline interpolation order")
                             ).set_prop("display", active=prop)
         param = RotateParam(_("Rotation"))
@@ -844,7 +850,7 @@ class ImageFT(ObjectFT):
                         spi.rotate(x, p.angle, reshape=p.reshape,
                                    order=p.order, mode=p.mode,
                                    cval=p.cval, prefilter=p.prefilter),
-                        param, suffix=lambda p: u"α=%.3f°, mode='%s'"\
+                        param, suffix=lambda p: "α=%.3f°, mode='%s'"\
                                                 % (p.angle, p.mode))
     
     def rotate_90(self):
@@ -863,7 +869,7 @@ class ImageFT(ObjectFT):
         rows = self._get_selected_rows()
         obj = self.objects[rows[0]]
         original_size = obj.data.shape[1], obj.data.shape[0]
-        from guiqwt.resizedialog import ResizeDialog
+        from guiqwt.widgets.resizedialog import ResizeDialog
         dlg = ResizeDialog(self.plot, new_size=original_size,
                            old_size=original_size,
                            text=_("Destination size:"))
@@ -872,16 +878,16 @@ class ImageFT(ObjectFT):
         boundaries = ('constant', 'nearest', 'reflect', 'wrap')
         prop = ValueProp(False)
         class ResizeParam(DataSet):
-            zoom = FloatItem(_(u"Zoom"), default=dlg.get_zoom())
-            mode = ChoiceItem(_(u"Mode"), zip(boundaries, boundaries),
+            zoom = FloatItem(_("Zoom"), default=dlg.get_zoom())
+            mode = ChoiceItem(_("Mode"), list(zip(boundaries, boundaries)),
                               default=boundaries[0])
             cval = FloatItem(_("cval"), default=0.,
-                             help=_(u"Value used for points outside the "
-                                    u"boundaries of the input if mode is "
-                                    u"'constant'"))
-            prefilter = BoolItem(_(u"Prefilter the input image"),
+                             help=_("Value used for points outside the "
+                                    "boundaries of the input if mode is "
+                                    "'constant'"))
+            prefilter = BoolItem(_("Prefilter the input image"),
                                  default=True).set_prop("display", store=prop)
-            order = IntItem(_(u"Order"), default=3, min=0, max=5,
+            order = IntItem(_("Order"), default=3, min=0, max=5,
                             help=_("Spline interpolation order")
                             ).set_prop("display", active=prop)
         param = ResizeParam(_("Resize"))
@@ -890,7 +896,7 @@ class ImageFT(ObjectFT):
                         spi.interpolation.zoom(x, p.zoom, order=p.order,
                                                mode=p.mode, cval=p.cval,
                                                prefilter=p.prefilter),
-                        param, suffix=lambda p: u"zoom=%.3f" % p.zoom)
+                        param, suffix=lambda p: "zoom=%.3f" % p.zoom)
                         
     def extract_roi(self):
         class ROIParam(DataSet):
@@ -901,7 +907,7 @@ class ImageFT(ObjectFT):
         param = ROIParam(_("ROI extraction"))
         self.compute_11("ROI", lambda x, p:
                         x.copy()[p.row1:p.row2, p.col1:p.col2],
-                        param, suffix=lambda p: u"rows=%d:%d,cols=%d:%d" 
+                        param, suffix=lambda p: "rows=%d:%d,cols=%d:%d" 
                         % (p.row1, p.row2, p.col1, p.col2))
     
     def swap_axes(self):
@@ -915,11 +921,11 @@ class ImageFT(ObjectFT):
         try:
             robj.data = flatfield(self.objects[rows[0]].data,
                                   self.objects[rows[1]].data)
-        except Exception, msg:
+        except Exception as msg:
             import traceback
             traceback.print_exc()
             QMessageBox.critical(self.parent(), APP_NAME,
-                                 _(u"Error:")+"\n%s" % str(msg))
+                                 _("Error:")+"\n%s" % str(msg))
             return
         self.add_object(robj)
         
@@ -930,21 +936,21 @@ class ImageFT(ObjectFT):
             b = FloatItem('b', default=0.)
         param = CalibrateParam(_("Linear calibration"), "y = a.x + b")
         self.compute_11("LinearCal", lambda x, p: p.a*x+p.b, param,
-                        suffix=lambda p: u"z=%s*z+%s" % (p.a, p.b))
+                        suffix=lambda p: "z=%s*z+%s" % (p.a, p.b))
     
     def compute_threshold(self):
         class ThresholdParam(DataSet):
-            value = FloatItem(_(u"Threshold"))
+            value = FloatItem(_("Threshold"))
         self.compute_11("Threshold", lambda x, p: np.clip(x, p.value, x.max()),
                         ThresholdParam(_("Thresholding")),
-                        suffix=lambda p: u"min=%s lsb" % p.value)
+                        suffix=lambda p: "min=%s lsb" % p.value)
                         
     def compute_clip(self):
         class ClipParam(DataSet):
-            value = FloatItem(_(u"Clipping value"))
+            value = FloatItem(_("Clipping value"))
         self.compute_11("Clip", lambda x, p: np.clip(x, x.min(), p.value),
                         ClipParam(_("Clipping")),
-                        suffix=lambda p: u"max=%s lsb" % p.value)
+                        suffix=lambda p: "max=%s lsb" % p.value)
                         
     def compute_wiener(self):
         import scipy.signal as sps
@@ -952,12 +958,12 @@ class ImageFT(ObjectFT):
     
     def compute_gaussian(self):
         class GaussianParam(DataSet):
-            sigma = FloatItem(u"σ", default=1.)
+            sigma = FloatItem("σ", default=1.)
         param = GaussianParam(_("Gaussian filter"))
         import scipy.ndimage as spi
         self.compute_11("GaussianFilter",
                         lambda x, p: spi.gaussian_filter(x, p.sigma), param,
-                        suffix=lambda p: u"σ=%.3f pixels" % p.sigma)
+                        suffix=lambda p: "σ=%.3f pixels" % p.sigma)
                          
     def compute_fft(self):
         self.compute_11("FFT", np.fft.fft2)
@@ -988,8 +994,8 @@ class ImageFT(ObjectFT):
             image.data = np.empty(shape, dtype=dtype)
         elif imagenew.type == 'rand':
             data = np.random.rand(*shape)
-            from guiqwt.io import set_dynamic_range_from_dtype
-            image.data = set_dynamic_range_from_dtype(data, dtype)
+            from guiqwt import io
+            image.data = io.scale_data_to_dtype(data, dtype)
         self.add_object(image)
     
     def open_image(self):
@@ -1037,7 +1043,7 @@ class DockablePlotWidget(DockableWidget):
         return self.plotwidget.plot
         
     def setup(self):
-        title = unicode(self.toolbar.windowTitle())
+        title = to_text_string(self.toolbar.windowTitle())
         self.plotwidget.add_toolbar(self.toolbar, title)
         if isinstance(self.plotwidget, ImageWidget):
             self.plotwidget.register_all_image_tools()
@@ -1129,7 +1135,7 @@ class MainWindow(QMainWindow):
                               _("Signals"))
         self.tabwidget.addTab(self.imageft, get_icon('image.png'),
                               _("Images"))
-        self.add_dockwidget(self.tabwidget, _(u"Main panel"))
+        self.add_dockwidget(self.tabwidget, _("Main panel"))
 #        self.setCentralWidget(self.tabwidget)
         self.curve_dock = self.add_dockwidget(self.curvewidget,
                                               title=_("Curve plotting panel"))
@@ -1169,7 +1175,7 @@ class MainWindow(QMainWindow):
         
         # View menu
         self.view_menu = view_menu = self.createPopupMenu()
-        view_menu.setTitle(_(u"&View"))
+        view_menu.setTitle(_("&View"))
         self.menuBar().addMenu(view_menu)
         
         # Help menu
@@ -1193,7 +1199,7 @@ class MainWindow(QMainWindow):
                   "os, sys, os.path as osp, time, "\
                   "numpy as np, scipy.signal as sps, scipy.ndimage as spi"
             self.console = DockableConsole(self, namespace=ns, message=msg)
-            self.add_dockwidget(self.console, _(u"Console"))
+            self.add_dockwidget(self.console, _("Console"))
             self.connect(self.console.interpreter.widget_proxy,
                          SIGNAL("new_prompt(QString)"),
                          lambda txt: self.refresh_lists())
