@@ -22,7 +22,8 @@ from guiqwt.config import _
 from guiqwt.plot import ImageDialog
 from guiqwt.builder import make
 from guiqwt.tools import OpenImageTool
-from guiqwt.io import imagefile_to_array
+from guiqwt import io
+
 
 class ImageParam(DataSet):
     title = StringItem(_("Title"))
@@ -43,7 +44,7 @@ class FilterParam(DataSet):
     
 class ExampleDialog(ImageDialog):
     def __init__(self, wintitle=_("Example dialog box"),
-                 icon="guidata.png", options=dict(show_contrast=True),
+                 icon="guidata.svg", options=dict(show_contrast=True),
                  edit=False):
         self.filter_gbox = None
         self.data = None
@@ -74,7 +75,7 @@ class ExampleDialog(ImageDialog):
         
     def open_image(self, filename):
         """Opening image *filename*"""
-        self.data = imagefile_to_array(filename, to_grayscale=True)
+        self.data = io.imread(filename, to_grayscale=True)
         self.show_data(self.data)
         param = ImageParam()
         param.title = filename
