@@ -24,12 +24,13 @@ from guidata.dataset.qtwidgets import DataSetEditGroupBox
 from guidata.configtools import get_icon
 from guidata.qthelpers import create_action, add_actions, get_std_icon
 from guidata.utils import update_dataset
+from guidata.py3compat import to_text_string
 
 from guiqwt.config import _
 from guiqwt.plot import ImageWidget
 from guiqwt.builder import make
 from guiqwt.signals import SIG_LUT_CHANGED
-from guiqwt.io import imagefile_to_array, IMAGE_LOAD_FILTERS
+from guiqwt import io
 
 APP_NAME = _("Application example")
 VERSION = '1.0.0'
@@ -148,8 +149,8 @@ class CentralWidget(QSplitter):
     
     def add_image_from_file(self, filename):
         image = ImageParam()
-        image.title = unicode(filename)
-        image.data = imagefile_to_array(filename, to_grayscale=True)
+        image.title = to_text_string(filename)
+        image.data = io.imread(filename, to_grayscale=True)
         image.height, image.width = image.data.shape
         self.add_image(image)
 
@@ -230,7 +231,7 @@ class MainWindow(QMainWindow):
         saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
         sys.stdout = None
         filename, _filter = getopenfilename(self, _("Open"), "",
-                                            IMAGE_LOAD_FILTERS)
+                                            io.iohandler.get_filters('load'))
         sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
         if filename:
             self.mainwidget.add_image_from_file(filename)
