@@ -238,6 +238,8 @@ Reference
 #TODO: z(long-terme) à partir d'une sélection rectangulaire sur une image
 #      afficher un ArrayEditor montrant les valeurs de la zone sélectionnée
 
+from __future__ import unicode_literals
+
 try:
     # PyQt4 4.3.3 on Windows (static DLLs) with py2exe installed:
     # -> pythoncom must be imported first, otherwise py2exe's boot_com_servers
@@ -247,24 +249,29 @@ try:
 except ImportError:
     pass
 
-import sys, numpy as np, weakref, os.path as osp
+import sys
+import numpy as np
+import weakref
+import os.path as osp
 
-from guidata.qt.QtCore import Qt, QObject, SIGNAL
+from guidata.qt.QtCore import Qt, QObject, SIGNAL, QPoint
 from guidata.qt.QtGui import (QMenu, QActionGroup, QPrinter, QMessageBox,
-                              QPrintDialog, QFont, QAction, QToolButton)
+                              QPrintDialog, QFont, QAction, QToolButton,
+                              QKeySequence)
 from guidata.qt.compat import getsavefilename, getopenfilename
 
 from guidata.qthelpers import get_std_icon, add_actions, add_separator
 from guidata.configtools import get_icon
-from guidata.utils import is_module_available
 from guidata.dataset.datatypes import DataSet
 from guidata.dataset.dataitems import BoolItem, FloatItem
+from guidata.py3compat import is_text_string, to_text_string
 
 #Local imports
 from guiqwt.transitional import QwtPlotPrintFilter
 from guiqwt.config import _
 from guiqwt.events import (setup_standard_tool_filter, ObjectHandler,
-                           KeyEventMatch, QtDragHandler, ZoomRectHandler,
+                           KeyEventMatch, StandardKeyMatch,
+                           QtDragHandler, ZoomRectHandler,
                            RectangularSelectionHandler, ClickHandler)
 from guiqwt.shapes import (Axes, RectangleShape, Marker, PolygonShape,
                            EllipseShape, SegmentShape, PointShape,
@@ -274,14 +281,14 @@ from guiqwt.annotations import (AnnotatedRectangle, AnnotatedCircle,
                                 AnnotatedPoint, AnnotatedObliqueRectangle)
 from guiqwt.colormap import get_colormap_list, get_cmap, build_icon_from_cmap
 from guiqwt.interfaces import (IColormapImageItemType, IPlotManager,
-                               IVoiImageItemType, IExportROIImageItemType,
-                               IStatsImageItemType, ICurveItemType)
+                               IVoiImageItemType, IStatsImageItemType,
+                               ICurveItemType)
 from guiqwt.signals import (SIG_VISIBILITY_CHANGED, SIG_CLICK_EVENT,
                             SIG_START_TRACKING, SIG_STOP_NOT_MOVING,
                             SIG_STOP_MOVING, SIG_MOVE, SIG_END_RECT,
                             SIG_VALIDATE_TOOL, SIG_ITEMS_CHANGED,
-                            SIG_ITEM_SELECTION_CHANGED, SIG_ITEM_REMOVED,
-                            SIG_APPLIED_MASK_TOOL, SIG_TOOL_JOB_FINISHED)
+                            SIG_ITEM_SELECTION_CHANGED, SIG_APPLIED_MASK_TOOL,
+                            SIG_TOOL_JOB_FINISHED)
 from guiqwt.panels import ID_XCS, ID_YCS, ID_OCS, ID_ITEMLIST, ID_CONTRAST
 
 
@@ -293,7 +300,7 @@ class GuiTool(QObject):
     """Base class for interactive tool applying on a plot"""
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
         """Constructor"""
-        QObject.__init__(self)
+        super(GuiTool, self).__init__()
         assert IPlotManager in manager.__implements__
         self.manager = manager
         self.parent_tool = None
@@ -367,7 +374,8 @@ class InteractiveTool(GuiTool):
     SWITCH_TO_DEFAULT_TOOL = False # switch to default tool when finished
 
     def __init__(self, manager, toolbar_id=DefaultToolbarID,
-                 title=None, icon=None, tip=None):
+                 title=None, icon=None, tip=None,
+                 switch_to_default_tool=None):
         if title is not None:
             self.TITLE = title
         if icon is not None:
@@ -378,7 +386,9 @@ class InteractiveTool(GuiTool):
         # Starting state for every plotwidget we can act upon
         self.start_state = {}
         
-        if self.SWITCH_TO_DEFAULT_TOOL:
+        if switch_to_default_tool is None:
+            switch_to_default_tool = self.SWITCH_TO_DEFAULT_TOOL
+        if switch_to_default_tool:
             self.connect(self, SIG_TOOL_JOB_FINISHED,
                          self.manager.activate_default_tool)
                         
@@ -420,7 +430,7 @@ class InteractiveTool(GuiTool):
 
     def activate(self):
         """Activate tool"""
-        for baseplot, start_state in self.start_state.items():
+        for baseplot, start_state in list(self.start_state.items()):
             baseplot.filter.set_state(start_state, None)
         self.action.setChecked(True)
         self.manager.set_active_tool(self)
@@ -454,8 +464,7 @@ class SelectTool(InteractiveTool):
                          KeyEventMatch((Qt.Key_Enter, Qt.Key_Return,
                                         Qt.Key_Space)),
                          self.validate, start_state)
-        filter.add_event(start_state,
-                         KeyEventMatch(((Qt.Key_A, Qt.ControlModifier),)),
+        filter.add_event(start_state, StandardKeyMatch(QKeySequence.SelectAll),
                          self.select_all_items, start_state)
         return setup_standard_tool_filter(filter, start_state)
 
@@ -473,9 +482,11 @@ class SelectPointTool(InteractiveTool):
     
     def __init__(self, manager, mode="reuse", on_active_item=False,
                  title=None, icon=None, tip=None, end_callback=None,
-                 toolbar_id=DefaultToolbarID, marker_style=None):
+                 toolbar_id=DefaultToolbarID, marker_style=None,
+                 switch_to_default_tool=None):
         super(SelectPointTool, self).__init__(manager, toolbar_id,
-                                              title=title, icon=icon, tip=tip)
+                              title=title, icon=icon, tip=tip,
+                              switch_to_default_tool=switch_to_default_tool)
         assert mode in ("reuse", "create")
         self.mode = mode
         self.end_callback = end_callback
@@ -551,9 +562,11 @@ class MultiLineTool(InteractiveTool):
     CURSOR = Qt.ArrowCursor
 
     def __init__(self, manager, handle_final_shape_cb=None, shape_style=None,
-                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None):
+                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None,
+                 switch_to_default_tool=None):
         super(MultiLineTool, self).__init__(manager, toolbar_id,
-                                            title=title, icon=icon, tip=tip)
+                                title=title, icon=icon, tip=tip,
+                                switch_to_default_tool=switch_to_default_tool)
         self.handle_final_shape_cb = handle_final_shape_cb
         self.shape = None
         self.current_handle = None
@@ -570,7 +583,7 @@ class MultiLineTool(InteractiveTool):
         self.current_handle = None
     
     def create_shape(self, filter, pt):
-        self.shape = PolygonShape([], closed=False)
+        self.shape = PolygonShape(closed=False)
         filter.plot.add_item_with_z_offset(self.shape, SHAPE_Z_OFFSET)
         self.shape.setVisible(True)
         self.shape.set_style(self.shape_style_sect, self.shape_style_key)
@@ -588,7 +601,7 @@ class MultiLineTool(InteractiveTool):
                                          Qt.Key_Space) ),
                          self.validate, start_state)
         filter.add_event(start_state,
-                         KeyEventMatch( (Qt.Key_Backspace,Qt.Key_Escape,) ),
+                         KeyEventMatch( (Qt.Key_Backspace, Qt.Key_Escape,) ),
                          self.cancel_point, start_state)
         self.connect(handler, SIG_START_TRACKING, self.mouse_press)
         self.connect(handler, SIG_MOVE, self.move)
@@ -676,10 +689,11 @@ class LabelTool(InteractiveTool):
     SWITCH_TO_DEFAULT_TOOL = True
     
     def __init__(self, manager, handle_label_cb=None, label_style=None,
-                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None):
+                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None,
+                 switch_to_default_tool=None):
         self.handle_label_cb = handle_label_cb
-        InteractiveTool.__init__(self, manager, toolbar_id,
-                                 title=title, icon=icon, tip=tip)
+        super(LabelTool, self).__init__(manager, toolbar_id, title=title,
+            icon=icon, tip=tip, switch_to_default_tool=switch_to_default_tool)
         if label_style is not None:
             self.label_style_sect = label_style[0]
             self.label_style_key = label_style[1]
@@ -724,10 +738,13 @@ class RectangularActionTool(InteractiveTool):
     SHAPE_STYLE_KEY = "shape/drag"
     AVOID_NULL_SHAPE = False
     def __init__(self, manager, func, shape_style=None,
-                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None):
+                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None,
+                 fix_orientation=False, switch_to_default_tool=None):
         self.action_func = func
-        InteractiveTool.__init__(self, manager, toolbar_id,
-                                 title=title, icon=icon, tip=tip)
+        self.fix_orientation = fix_orientation
+        super(RectangularActionTool, self).__init__(manager, toolbar_id,
+                                 title=title, icon=icon, tip=tip,
+                                 switch_to_default_tool=switch_to_default_tool)
         if shape_style is not None:
             self.shape_style_sect = shape_style[0]
             self.shape_style_key = shape_style[1]
@@ -735,6 +752,7 @@ class RectangularActionTool(InteractiveTool):
             self.shape_style_sect = self.SHAPE_STYLE_SECT
             self.shape_style_key = self.SHAPE_STYLE_KEY
         self.last_final_shape = None
+        self.switch_to_default_tool = switch_to_default_tool
         
     def get_last_final_shape(self):
         if self.last_final_shape is not None:
@@ -779,20 +797,29 @@ class RectangularActionTool(InteractiveTool):
 
     def end_rect(self, filter, p0, p1):
         plot = filter.plot
-        self.action_func(plot, p0, p1)
+        if self.fix_orientation:
+            left, right = min(p0.x(), p1.x()), max(p0.x(), p1.x())
+            top, bottom = min(p0.y(), p1.y()), max(p0.y(), p1.y())
+            self.action_func(plot, QPoint(left, top), QPoint(right, bottom))
+        else:
+            self.action_func(plot, p0, p1)
         self.emit(SIG_TOOL_JOB_FINISHED)
+        if self.switch_to_default_tool:
+            shape = self.get_last_final_shape()
+            plot.set_active_item(shape)
 
 
 class RectangularShapeTool(RectangularActionTool):
     TITLE = None
     ICON = None
-    def __init__(self, manager, setup_shape_cb=None, handle_final_shape_cb=None,
-                 shape_style=None, toolbar_id=DefaultToolbarID,
-                 title=None, icon=None, tip=None):
-        RectangularActionTool.__init__(self, manager,
-                                       self.add_shape_to_plot, shape_style,
-                                       toolbar_id=toolbar_id,
-                                       title=title, icon=icon, tip=tip)
+    def __init__(self, manager, setup_shape_cb=None,
+                 handle_final_shape_cb=None, shape_style=None,
+                 toolbar_id=DefaultToolbarID, title=None, icon=None, tip=None,
+                 switch_to_default_tool=None):
+        super(RectangularShapeTool, self).__init__(manager,
+                       self.add_shape_to_plot, shape_style,
+                       toolbar_id=toolbar_id, title=title, icon=icon, tip=tip,
+                       switch_to_default_tool=switch_to_default_tool)
         self.setup_shape_cb = setup_shape_cb
         self.handle_final_shape_cb = handle_final_shape_cb
         
@@ -873,7 +900,7 @@ class PlaceAxesTool(RectangularShapeTool):
     ICON = "gtaxes.png"
     SHAPE_STYLE_KEY = "shape/axes"
     def create_shape(self):
-        shape = Axes( (0,1), (1,1), (0,0) )
+        shape = Axes( (0, 1), (1, 1), (0, 0) )
         self.set_shape_style(shape)
         return shape, 0, 2
 
@@ -953,7 +980,11 @@ class ImageStatsTool(RectangularShapeTool):
                                 handle_final_shape_cb, shape_style, toolbar_id,
                                 title, icon, tip)
         self._last_item = None
-        
+
+    def get_last_item(self):
+        if self._last_item is not None:
+            return self._last_item()
+
     def create_shape(self):
         return ImageStatsRectangle(0, 0, 1, 1), 0, 2
         
@@ -968,7 +999,7 @@ class ImageStatsTool(RectangularShapeTool):
         if plot is not None:
             plot.unselect_all()
             plot.set_active_item(shape)
-            shape.set_image_item(self._last_item)
+            shape.set_image_item(self.get_last_item())
 
     def handle_final_shape(self, shape):
         super(ImageStatsTool, self).handle_final_shape(shape)
@@ -977,8 +1008,8 @@ class ImageStatsTool(RectangularShapeTool):
     def get_associated_item(self, plot):
         items = plot.get_selected_items(item_type=IStatsImageItemType)
         if len(items) == 1:
-            self._last_item = items[0]
-        return self._last_item
+            self._last_item = weakref.ref(items[0])
+        return self.get_last_item()
         
     def update_status(self, plot):
         if update_image_tool_status(self, plot):
@@ -1068,7 +1099,7 @@ class RectZoomTool(InteractiveTool):
         return setup_standard_tool_filter(filter, start_state)
     
     def get_shape(self):
-        shape = RectangleShape(0,0,1,1)
+        shape = RectangleShape(0, 0, 1, 1)
         shape.set_style("plot", "shape/rectzoom")
         return shape, 0, 2
 
@@ -1077,9 +1108,10 @@ class BaseCursorTool(InteractiveTool):
     TITLE = None
     ICON = None
     def __init__(self, manager, toolbar_id=DefaultToolbarID,
-                 title=None, icon=None, tip=None):
+                 title=None, icon=None, tip=None, switch_to_default_tool=None):
         super(BaseCursorTool, self).__init__(manager, toolbar_id,
-                                             title=title, icon=icon, tip=tip)
+                                 title=title, icon=icon, tip=tip,
+                                 switch_to_default_tool=switch_to_default_tool)
         self.shape = None
 
     def create_shape(self):
@@ -1161,7 +1193,11 @@ class SignalStatsTool(BaseCursorTool):
                                               icon=icon, tip=tip)
         self._last_item = None
         self.label = None
-        
+
+    def get_last_item(self):
+        if self._last_item is not None:
+            return self._last_item()
+
     def create_shape(self):
         from guiqwt.shapes import XRangeSelection
         return XRangeSelection(0, 0)
@@ -1174,11 +1210,14 @@ class SignalStatsTool(BaseCursorTool):
             from guiqwt.builder import make
             self.label = make.computations(self.shape, "TL",
               [
-               (curve, "%g &lt; y &lt; %g", lambda *args: (args[1].min(), args[1].max())),
+               (curve, "%g &lt; x &lt; %g", 
+                lambda *args: (args[0].min(), args[0].max())),
+               (curve, "%g &lt; y &lt; %g", 
+                lambda *args: (args[1].min(), args[1].max())),
                (curve, "&lt;y&gt;=%g", lambda *args: args[1].mean()),
-               (curve, u"σ(y)=%g", lambda *args: args[1].std()),
-               (curve, u"∑(y)=%g", lambda *args: np.trapz(args[1])),
-               (curve, u"∫ydx=%g", lambda *args: np.trapz(args[1], args[0])),
+               (curve, "σ(y)=%g", lambda *args: args[1].std()),
+               (curve, "∑(y)=%g", lambda *args: np.trapz(args[1])),
+               (curve, "∫ydx=%g", lambda *args: np.trapz(args[1], args[0])),
               ])
             self.label.attach(plot)
             self.label.setZ(plot.get_max_z()+1)
@@ -1193,8 +1232,8 @@ class SignalStatsTool(BaseCursorTool):
     def get_associated_item(self, plot):
         items = plot.get_selected_items(item_type=ICurveItemType)
         if len(items) == 1:
-            self._last_item = items[0]
-        return self._last_item
+            self._last_item = weakref.ref(items[0])
+        return self.get_last_item()
         
     def update_status(self, plot):
         item = self.get_associated_item(plot)
@@ -1218,7 +1257,7 @@ class CommandTool(GuiTool):
     def __init__(self, manager, title, icon=None, tip=None,
                  toolbar_id=DefaultToolbarID):
         self.title = title
-        if icon and isinstance(icon, basestring):
+        if icon and is_text_string(icon):
             self.icon = get_icon(icon)
         else:
             self.icon = icon
@@ -1462,7 +1501,7 @@ class ItemListPanelTool(PanelTool):
 
 class SaveAsTool(CommandTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        super(SaveAsTool,self).__init__(manager, _("Save as..."),
+        super(SaveAsTool, self).__init__(manager, _("Save as..."),
                                         get_std_icon("DialogSaveButton", 16),
                                         toolbar_id=toolbar_id)
     def activate_command(self, plot, checked):
@@ -1483,7 +1522,7 @@ class SaveAsTool(CommandTool):
 
 class CopyToClipboardTool(CommandTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        super(CopyToClipboardTool,self).__init__(manager,
+        super(CopyToClipboardTool, self).__init__(manager,
                                         _("Copy to clipboard"),
                                         get_icon('copytoclipboard.png'),
                                         toolbar_id=toolbar_id)
@@ -1491,33 +1530,26 @@ class CopyToClipboardTool(CommandTool):
         """Activate tool"""
         plot.copy_to_clipboard()
 
-    
+
 def save_snapshot(plot, p0, p1):
     """
     Save rectangular plot area
     p0, p1: resp. top left and bottom right points (QPoint objects)
     """
-    from guiqwt.image import TrImageItem, get_image_from_plot, get_plot_qrect
-    from guiqwt.io import (array_to_imagefile, array_to_dicomfile,
-                           MODE_INTENSITY_U8, set_dynamic_range_from_dtype)
-    items = plot.get_items(item_type=IExportROIImageItemType)
-    src_qrect = get_plot_qrect(plot, p0, p1)
-    src_x, src_y, src_w, src_h = src_qrect.getRect()
-    items = [it for it in items if src_qrect.intersects(it.boundingRect())]
+    from guiqwt.image import (get_image_from_plot, get_plot_qrect,
+                              get_items_in_rectangle,
+                              compute_trimageitems_original_size)
+    from guiqwt import io
+    items = get_items_in_rectangle(plot, p0, p1)
     if not items:
         QMessageBox.critical(plot, _("Rectangle snapshot"),
                  _("There is no supported image item in current selection."))
         return
-    original_size = (src_w, src_h)
-    trparams = [item.get_transform() for item in items
-                if isinstance(item, TrImageItem)]
-    if trparams:
-        dx_max = max([dx for _x, _y, _angle, dx, _dy, _hf, _vf in trparams])
-        dy_max = max([dy for _x, _y, _angle, _dx, dy, _hf, _vf in trparams])
-        original_size = (src_w/dx_max, src_h/dy_max)
+    src_x, src_y, src_w, src_h = get_plot_qrect(plot, p0, p1).getRect()
+    original_size = compute_trimageitems_original_size(items, src_w, src_h)
     screen_size = (p1.x()-p0.x()+1, p1.y()-p0.y()+1)
     
-    from guiqwt.resizedialog import ResizeDialog
+    from guiqwt.widgets.resizedialog import ResizeDialog
     dlg = ResizeDialog(plot, new_size=screen_size, old_size=original_size,
                        text=_("Destination size:"))
     if not dlg.exec_():
@@ -1529,53 +1561,63 @@ def save_snapshot(plot, p0, p1):
         _levels = BeginGroup(_("Image levels adjustments"))
         apply_contrast = BoolItem(_("Apply contrast settings"),
                                   default=False)
+        apply_interpolation = BoolItem(_("Apply interpolation algorithm"),
+                                       default=True)
         norm_range = BoolItem(_("Scale levels to maximum range"),
                               default=False)
         _end_levels = EndGroup(_("Image levels adjustments"))
         _multiple = BeginGroup(_("Superimposed images"))
-        add_images = ChoiceItem(_(u"If image B is behind image A, "
-                                  u"replace intersection by"),
-                               [(False, _(u"A")), (True, _(u"A+B"))],
+        add_images = ChoiceItem(_("If image B is behind image A, "
+                                  "replace intersection by"),
+                               [(False, "A"), (True, "A+B")],
                                default=None)
         _end_multiple = EndGroup(_("Superimposed images"))
     
     param = SnapshotParam(_("Rectangle snapshot"))
     if not param.edit(parent=plot):
         return
-        
-    data = get_image_from_plot(plot, p0, p1, dlg.width, dlg.height,
-                               apply_lut=param.apply_contrast,
-                               add_images=param.add_images)
-
-    dtype = None
-    for item in items:
-        if dtype is None or item.data.dtype.itemsize > dtype.itemsize:
-            dtype = item.data.dtype
-    if param.norm_range:
-        data = set_dynamic_range_from_dtype(data, dtype=dtype)
+    
+    if dlg.keep_original_size:
+        destw, desth = original_size
     else:
-        data = np.array(data, dtype=dtype)
-            
+        destw, desth = dlg.width, dlg.height
+    
+    try:
+        data = get_image_from_plot(plot, p0, p1, destw=destw, desth=desth,
+                               add_images=param.add_images,
+                               apply_lut=param.apply_contrast,
+                               apply_interpolation=param.apply_interpolation,
+                               original_resolution=dlg.keep_original_size)
+    
+        dtype = None
+        for item in items:
+            if dtype is None or item.data.dtype.itemsize > dtype.itemsize:
+                dtype = item.data.dtype
+        if param.norm_range:
+            data = io.scale_data_to_dtype(data, dtype=dtype)
+        else:
+            data = np.array(data, dtype=dtype)
+    except MemoryError:
+        mbytes = int(destw*desth*32./(8*1024**2))
+        QMessageBox.critical(plot, _("Memory error"),
+                             _("There is not enough memory left to process "
+                               "this %d x %d image (%d MB would be required)."
+                             ) % (destw, desth, mbytes))
+        return
     for model_item in items:
         model_fname = model_item.get_filename()
         if model_fname is not None and model_fname.lower().endswith(".dcm"):
             break
     else:
         model_fname = None
-    if is_module_available('dicom') and model_fname is not None:
-        formats = '%s (*.dcm)' % _("16-bits DICOM image")
-    else:
-        formats = ''
-    formats += '\n%s (*.tif *.tiff)' % _('16-bits TIFF image')
-    formats += '\n%s (*.png)' % _('8-bits PNG image')
-    fname, _f = getsavefilename(plot,  _("Save as"), _('untitled'), formats)
+    fname, _f = getsavefilename(plot,  _("Save as"), _('untitled'),
+                   io.iohandler.get_filters('save', data.dtype, template=True))
     _base, ext = osp.splitext(fname)
+    options = {}
     if not fname:
         return
     elif ext.lower() == ".png":
-        array_to_imagefile(data, fname, MODE_INTENSITY_U8, max_range=True)
-    elif ext.lower() in (".tif", ".tiff"):
-        array_to_imagefile(data, fname)
+        options.update(dict(dtype=np.uint8, max_range=True))
     elif ext.lower() == ".dcm":
         import dicom
         model_dcm = dicom.read_file(model_fname)
@@ -1593,23 +1635,54 @@ def save_snapshot(plot, p0, p1):
         new_ps_x = ps_x*src_w/(model_dx*dest_width)
         new_ps_y = ps_y*src_h/(model_dy*dest_height)
         setattr(model_dcm, ps_attr, [new_ps_x, new_ps_y])
-        
-        array_to_dicomfile(data, model_dcm, fname)
-    else:
-        raise RuntimeError(_("Unknown file extension"))
+        options.update(dict(template=model_dcm))
+    io.imwrite(fname, data, **options)
 
 class SnapshotTool(RectangularActionTool):
     SWITCH_TO_DEFAULT_TOOL = True
     TITLE = _("Rectangle snapshot")
     ICON = "snapshot.png"
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        RectangularActionTool.__init__(self, manager, save_snapshot,
-                                       toolbar_id=toolbar_id)
+        super(SnapshotTool, self).__init__(manager, save_snapshot,
+                                           toolbar_id=toolbar_id,
+                                           fix_orientation=True)
+
+
+class RotateCropTool(CommandTool):
+    """Rotate & Crop tool
+    
+    See :py:class:`guiqwt.rotatecrop.RotateCropDialog` dialog."""
+    def __init__(self, manager, toolbar_id=DefaultToolbarID, options=None):
+        super(RotateCropTool, self).__init__(manager,
+                        title=_("Rotate and crop"),
+                        icon=get_icon('rotate.png'), toolbar_id=toolbar_id)
+        self.options = options
+
+    def activate_command(self, plot, checked):
+        """Activate tool"""
+        from guiqwt.image import TrImageItem
+        from guiqwt.widgets.rotatecrop import RotateCropDialog
+        for item in plot.get_selected_items():
+            if isinstance(item, TrImageItem):
+                z = item.z()
+                plot.del_item(item)
+                dlg = RotateCropDialog(plot.parent(), options=self.options)
+                dlg.set_item(item)
+                ok = dlg.exec_()
+                plot.add_item(item, z=z)
+                if not ok:
+                    break
+
+    def update_status(self, plot):
+        from guiqwt.image import TrImageItem
+        status = any([isinstance(item, TrImageItem)
+                      for item in plot.get_selected_items()])
+        self.action.setEnabled(status)
 
 
 class PrintFilter(QwtPlotPrintFilter):
     def __init__(self):
-        QwtPlotPrintFilter.__init__(self)
+        super(PrintFilter, self).__init__()
 
     def color(self, c, item):
         if not (self.options() & QwtPlotPrintFilter.CanvasBackground):
@@ -1632,7 +1705,7 @@ class PrintFilter(QwtPlotPrintFilter):
 
 class PrintTool(CommandTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        super(PrintTool,self).__init__(manager, _("Print..."),
+        super(PrintTool, self).__init__(manager, _("Print..."),
                                        get_icon("print.png"),
                                        toolbar_id=toolbar_id)
     def activate_command(self, plot, checked):
@@ -1653,10 +1726,10 @@ class PrintTool(CommandTool):
 
 
 class OpenFileTool(CommandTool):
-    def __init__(self, manager, formats='*.*', toolbar_id=DefaultToolbarID):
-        CommandTool.__init__(self, manager, _("Open..."),
-                             get_std_icon("DialogOpenButton", 16),
-                             toolbar_id=toolbar_id)
+    def __init__(self, manager, title=_("Open..."), formats='*.*',
+                 toolbar_id=DefaultToolbarID):
+        super(OpenFileTool, self).__init__(manager, title,
+                  get_std_icon("DialogOpenButton", 16), toolbar_id=toolbar_id)
         self.formats = formats
         self.directory = ""
         
@@ -1666,7 +1739,7 @@ class OpenFileTool(CommandTool):
         filename, _f = getopenfilename(plot, _("Open"),
                                        self.directory, self.formats)
         sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
-        filename = unicode(filename)
+        filename = to_text_string(filename)
         if filename:
             self.directory = osp.dirname(filename)
         return filename
@@ -1680,40 +1753,38 @@ class OpenFileTool(CommandTool):
 
 class SaveItemsTool(CommandTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        CommandTool.__init__(self, manager, _("Save items"),
-                             get_std_icon("DialogSaveButton", 16),
-                             toolbar_id=toolbar_id)
+        super(SaveItemsTool, self).__init__(manager, _("Save items"),
+                  get_std_icon("DialogSaveButton", 16), toolbar_id=toolbar_id)
+
     def activate_command(self, plot, checked):
         """Activate tool"""
         fname, _f = getsavefilename(plot, _("Save items as"), _('untitled'),
                                     '%s (*.gui)' % _("guiqwt items"))
         if not fname:
             return
-        itemfile = file(fname, "wb")
+        itemfile = open(fname, "wb")
         plot.save_items(itemfile, selected=True)
 
 class LoadItemsTool(OpenFileTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        CommandTool.__init__(self, manager, _("Load items"),
-                             get_std_icon("DialogOpenButton", 16),
-                             toolbar_id=toolbar_id)
-        self.formats = '*.gui'
+        super(LoadItemsTool, self).__init__(manager, title=_("Load items"),
+                                      formats='*.gui', toolbar_id=toolbar_id)
 
     def activate_command(self, plot, checked):
         """Activate tool"""
         filename = self.get_filename(plot)
         if not filename:
             return
-        itemfile = file(filename, "rb")
+        itemfile = open(filename, "rb")
         plot.restore_items(itemfile)
         plot.replot()
 
 
 class OpenImageTool(OpenFileTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        from guiqwt.io import IMAGE_LOAD_FILTERS
-        OpenFileTool.__init__(self, manager, formats=IMAGE_LOAD_FILTERS,
-                              toolbar_id=toolbar_id)
+        from guiqwt import io
+        super(OpenImageTool, self).__init__(manager, title=_("Open image"),
+            formats=io.iohandler.get_filters('load'), toolbar_id=toolbar_id)
     
 
 class AxisScaleTool(CommandTool):
@@ -1742,7 +1813,7 @@ class AxisScaleTool(CommandTool):
         self.scale_menu = {("lin", "lin"): lin_lin, ("lin", "log"): lin_log,
                            ("log", "lin"): log_lin, ("log", "log"): log_log}
         for obj in (group, menu):
-           add_actions(obj, (lin_lin, lin_log, log_lin, log_log))
+            add_actions(obj, (lin_lin, lin_log, log_lin, log_log))
         return menu
      
     def update_status(self, plot):
@@ -1752,7 +1823,7 @@ class AxisScaleTool(CommandTool):
             xscale = plot.get_axis_scale(item.xAxis())
             yscale = plot.get_axis_scale(item.yAxis())
             active_scale = xscale, yscale
-        for scale_type, scale_action in self.scale_menu.items():
+        for scale_type, scale_action in list(self.scale_menu.items()):
             if item is None:
                 scale_action.setEnabled(False)
             else:
@@ -1774,7 +1845,7 @@ class AxisScaleTool(CommandTool):
 
 class HelpTool(CommandTool):
     def __init__(self, manager, toolbar_id=DefaultToolbarID):
-        super(HelpTool,self).__init__(manager, _("Help"),
+        super(HelpTool, self).__init__(manager, _("Help"),
                                       get_std_icon("DialogHelpButton", 16),
                                       toolbar_id=toolbar_id)
                                       
@@ -1846,12 +1917,12 @@ def export_curve_data(item):
     fname, _f = getsavefilename(plot, title, "", _("Text file")+" (*.txt)")
     if fname:
         try:
-            np.savetxt(unicode(fname), data, delimiter=',')
-        except RuntimeError, error:
+            np.savetxt(to_text_string(fname), data, delimiter=',')
+        except RuntimeError as error:
             QMessageBox.critical(plot, _("Export"),
                                  _("Unable to export item data.")+\
                                  "<br><br>"+_("Error message:")+"<br>"+\
-                                 str(error))
+                                 to_text_string(error))
 
 def export_image_data(item):
     """Export image item data to file"""
@@ -1862,7 +1933,7 @@ class ExportItemDataTool(ItemManipulationBaseTool):
     TITLE = _("Export data...")
     ICON = "export.png"
     def __init__(self, manager, toolbar_id=None):
-        super(ExportItemDataTool,self).__init__(manager, toolbar_id,
+        super(ExportItemDataTool, self).__init__(manager, toolbar_id,
                 curve_func=export_curve_data, image_func=export_image_data)
 
 def edit_curve_data(item):
@@ -1880,7 +1951,7 @@ def edit_curve_data(item):
         x, y = item_data
         data = np.array([x, y]).T
     from spyderlib.widgets.objecteditor import oedit
-    if oedit(data):
+    if oedit(data) is not None:
         if data.shape[1] > 2:
             if data.shape[1] == 3:
                 x, y, tmp = data.T
@@ -1905,13 +1976,13 @@ class EditItemDataTool(ItemManipulationBaseTool):
     TITLE = _("Edit data...")
     ICON = "arredit.png"
     def __init__(self, manager, toolbar_id=None):
-        super(EditItemDataTool,self).__init__(manager, toolbar_id,
+        super(EditItemDataTool, self).__init__(manager, toolbar_id,
                 curve_func=edit_curve_data, image_func=edit_image_data)
 
 
 class ItemCenterTool(CommandTool):
     def __init__(self, manager, toolbar_id=None):
-        super(ItemCenterTool,self).__init__(manager, _("Center items"),
+        super(ItemCenterTool, self).__init__(manager, _("Center items"),
                                             "center.png", toolbar_id=toolbar_id)
         
     def get_supported_items(self, plot):
@@ -1940,7 +2011,7 @@ class ItemCenterTool(CommandTool):
 
 class DeleteItemTool(CommandTool):
     def __init__(self, manager, toolbar_id=None):
-        super(DeleteItemTool,self).__init__(manager, _("Remove"), "delete.png",
+        super(DeleteItemTool, self).__init__(manager, _("Remove"), "trash.png",
                                             toolbar_id=toolbar_id)
         
     def get_removable_items(self, plot):
@@ -1961,41 +2032,12 @@ class DeleteItemTool(CommandTool):
                                      QMessageBox.Yes | QMessageBox.No)
         if answer == QMessageBox.Yes:
             plot.del_items(items)
-            for item in items:
-                plot.emit(SIG_ITEM_REMOVED, item)
             plot.replot()
-
-
-class DuplicateCurveTool(CommandTool):
-    def __init__(self, manager, toolbar_id=None):
-        super(DuplicateCurveTool,self).__init__(manager, _("Duplicate"),
-                                                "copy.png",
-                                                toolbar_id=toolbar_id)
-    def update_status(self, plot):
-        self.set_status_active_item()
-            
-    def activate_command(self, plot, checked):
-        """Activate tool"""
-        plot.duplicate_active_curve()
-
-
-class DeleteCurveTool(CommandTool):
-    def __init__(self, manager, toolbar_id=None):
-        super(DeleteCurveTool,self).__init__(manager, _("Remove"),
-                                             "delete.png",
-                                             toolbar_id=toolbar_id)
-
-    def update_status(self, plot):
-        self.set_status_active_item()
-            
-    def activate_command(self, plot, checked):
-        """Activate tool"""
-        plot.remove_active_curve()
         
         
 class FilterTool(CommandTool):
     def __init__(self, manager, filter, toolbar_id=None):
-        super(FilterTool, self).__init__(manager, unicode(filter.name),
+        super(FilterTool, self).__init__(manager, to_text_string(filter.name),
                                          toolbar_id=toolbar_id)
         self.filter = filter
 
@@ -2200,16 +2242,16 @@ class ImageMaskTool(CommandTool):
     def create_shapes_from_masked_areas(self):
         plot = self.get_active_plot()
         self._mask_shapes[plot] = []
-        masked_areas = self.masked_image.get_masked_areas()
-        for geometry, x0, y0, x1, y1, inside in masked_areas:
-            if geometry == 'rectangular':
-                shape = RectangleShape(x0, y0, x1, y1)
+        for area in self.masked_image.get_masked_areas():
+            if area.geometry == 'rectangular':
+                shape = RectangleShape(area.x0, area.y0, area.x1, area.y1)
                 self.masked_image.align_rectangular_shape(shape)
             else:
-                shape = EllipseShape(x0, .5*(y0+y1), x1, .5*(y0+y1))
+                shape = EllipseShape(area.x0, .5*(area.y0+area.y1),
+                                     area.x1, .5*(area.y0+area.y1))
             shape.set_style("plot", "shape/mask")
             shape.set_private(True)
-            self._mask_shapes[plot] += [(shape, inside)]
+            self._mask_shapes[plot] += [(shape, area.inside)]
             plot.blockSignals(True)
             plot.add_item(shape)
             plot.blockSignals(False)
