@@ -34,8 +34,8 @@ setup(
                 "script": "simpledialog.pyw",
                 "dest_base": "simpledialog",
                 "version": "1.0.0",
-                "company_name": u"CEA",
-                "copyright": u"Copyright © 2010 CEA - Pierre Raybaut",
+                "company_name": "CEA",
+                "copyright": "Copyright © 2010 CEA - Pierre Raybaut",
                 "name": "Simple dialog box",
                 "description": "Simple dialog box",
                 },],
